//
//  JapanPostBarcode.m
//  BarcodePod
//
//  Created by 佐藤 昭 on Mon Jun 30 2003.
//  Copyright (c) 2003 SatoAkira. All rights reserved.
//

#import "JapanPostBarcode.h"

@implementation JapanPostBarcode

#define MILLIMETERPERPOINT 0.3527777777777777 // base72.0 //
#define JP_DEFAULTHEIGHT 10.0
#define JP_MINIMUMLENGTH 7
#define JP_DESCENDER_BOTTOM 0.0	// 最下段基準線(mm) //
#define JP_TRACK_BOTTOM 1.2		// 中段下基準線(mm) //
#define JP_TRACK_TOP 2.4		// 中段上基準線(mm) //
#define JP_ASCENDER_TOP 3.6		// 最上段基準線(mm) //
#define JP_ASCENDER_MASK (0x222)
#define JP_DESCENDER_MASK (0x111)

// -----------------------------------------------------------------------------------
// Constants
// -----------------------------------------------------------------------------------

// Each hex constant represents a 4-bar code of the barcode corresponding to a single number or letter.  The digits represent bars in this fashion: //
//     0 - タイミングバー (JP_TRACK_BOTTOM to JP_TRACK_TOP) //
//     1 - 下セミロングバー (JP_DESCENDER_BOTTOM to JP_TRACK_TOP) //
//     2 - 上セミロングバー  (JP_TRACK_BOTTOM to JP_ASCENDER_TOP) //
//     3 - ロングバー  (JP_DESCENDER_BOTTOM to JP_ASCENDER_TOP) //
NSUInteger openBracketLength = 2;
NSUInteger openBracket = 0x31; // スタートコード //
NSUInteger closeBracketLength = 2;
NSUInteger closeBracket = 0x13; // ストップコード //
NSString *barFormat = @"101010101010101010101010";
NSUInteger numberLength = 3;
NSUInteger numberCodesOfJapan[] = {
	0x030, // -     //
	0x003, // CC7 . //
	0x333, // CC8 / //
    0x300, // 0     //
    0x330, // 1     //
    0x312, // 2     //
    0x132, // 3     //
    0x321, // 4     //
    0x303, // 5     //
    0x123, // 6     //
    0x231, // 7     //
    0x213, // 8     //
    0x033, // 9     //
    0x120, // CC1 : //
    0x102, // CC2 ; //
    0x210, // CC3 < //
    0x012, // CC4 = //
    0x201, // CC5 > //
    0x021  // CC6 ? //
};

double japanpost_barTop(NSUInteger hexDigit,double size) { // mm単位で返す。 //
	return (hexDigit & JP_ASCENDER_MASK) ? JP_ASCENDER_TOP * size / JP_DEFAULTHEIGHT : JP_TRACK_TOP * size / JP_DEFAULTHEIGHT;
}
double japanpost_barBottom(NSUInteger hexDigit,double size) { // mm単位で返す。 //
	return (hexDigit & JP_DESCENDER_MASK) ? JP_DESCENDER_BOTTOM * size / JP_DEFAULTHEIGHT : JP_TRACK_BOTTOM * size / JP_DEFAULTHEIGHT;
}
NSUInteger japanpost_characterDescriptor(unichar character) {
	return numberCodesOfJapan[character - '-'];
}

NSUInteger japanpost_barDescriptor(NSUInteger descriptor,NSUInteger bar,NSUInteger bars)
{
	NSUInteger shift = (bars - 1 - (bar / 2)) * 4; // 4ビットずつで1つのバーを表す。bar / 2 番目 //
	NSUInteger mask = 0xF << shift; // 0xfは4ビット分 //

	return (descriptor & mask) >> shift;
}

- (NSUInteger)_barDescriptor:(NSUInteger)index
// barcode is [self initiator][self barcode][self terminator] //
// 2 * numberLength がほぼ定数に近い働きをしており、これを変数にすることが難しい。よって'A'~'Z'は2文字の扱いをした。 //
{
	NSUInteger bar;
    NSUInteger hexDigit;
    NSUInteger descriptor;
    NSUInteger contentLength = [japanpostContents length];

    if ([[self initiator] length] > index) {
		bar = index % [[self initiator] length];
		hexDigit = openBracket;
		descriptor = japanpost_barDescriptor(hexDigit,bar,openBracketLength);
	}
	else {
		if (index >= (contentLength + 1) * (2 * numberLength) + [[self initiator] length]) {
			bar = (index - [[self initiator] length] - (contentLength + 1) * (2 * numberLength)) % [[self terminator] length];
			hexDigit = closeBracket;
			descriptor = japanpost_barDescriptor(hexDigit,bar,closeBracketLength);
		}
		else {
			NSUInteger digit = (index - [[self initiator] length]) / (2 * numberLength); // 何文字目か //

			bar = (index - [[self initiator] length]) % (2 * numberLength); // 1文字の中の何番目のバーになるか //
			if (digit != contentLength) {
				hexDigit = japanpost_characterDescriptor( [[self japanpostContents] characterAtIndex:digit] );
				descriptor = japanpost_barDescriptor( hexDigit, bar,numberLength);
			}
			else { // last digit is checksum
				hexDigit = japanpost_characterDescriptor( [self checkDigit] );
				descriptor = japanpost_barDescriptor( hexDigit, bar,numberLength);
			}
		}
	}
	return descriptor;
}

- (NSString *)japanpostContents {
	return japanpostContents;
}

// override //

- (id)initWithContent:(NSString *)inContent printsCaption:(BOOL)inPrints andBarWidth:(CGFloat)inBarWidth andHeight:(CGFloat)inHeight andFontSize:(CGFloat)inFontSize andCheckDigit:(char)inDigit
// inPrints,inBarWidth,inFontSize,inDigitは意味を持たない。 //
{
	if ((nil != inContent) && (nil != (self = [super init]))) {
		[self setContent:inContent];
		if ((8.0 <= inHeight) && (11.5 >= inHeight))
			[self setHeight:inHeight];
		else
			[self setHeight:JP_DEFAULTHEIGHT];
	}
	return self;
}

- (void)setHeight:(CGFloat)inHeight
// inHeight:ポイント単位 //
{
	if (8.5 > inHeight)
		height = 8.5;
	else {
		if (11.5 < inHeight)
			height = 11.5;
		else
			height = inHeight;
	}
	barWidth = (CGFloat)((double)height / (double)6.0);
	[self calculateWidth];
}

- (void)setBarWidth:(CGFloat)inBarWidth
// inBarWidth:point単位 //
{
	if (1.3333333333333333 > inBarWidth) // 8.0 / 6.0 //
		barWidth = 1.3333333333333333;
	else {
		if (1.9166666666667 < inBarWidth) // 11.5 / 6.0 //
			barWidth = 1.9166666666667;
		else
			barWidth = inBarWidth;
	}
	height = (CGFloat)((double)6.0 * (double)barWidth);
	NSLog(@"setBarWidth");
	[self calculateWidth];
}

- (void)generateChecksum
{
	NSUInteger i,cd;
	unichar uChar,character;
	NSUInteger checkValue = 0;

	for (i = 0; i < [japanpostContents length]; i++) {
		uChar = [japanpostContents characterAtIndex:i];
		if ('-' == uChar)
			checkValue += 10; // 10 //
		else {
			if ((',' == uChar) || ('/' == uChar))
				checkValue += (uChar - '-') + 16; // 17 or 18 //
			else {
				if (('0' <= uChar) && ('9' >= uChar))
					checkValue += uChar - '0'; // 0 ~ 9 //
				else {
					checkValue += uChar - ':' + 11; // 11 ~ 16 //
				}
			}
		}
	}
	cd = 19 * (checkValue / 19 + 1) - checkValue;
	if (10 > cd)
		character = '0' + cd;
	else {
		if (10 == cd)
			character = '-';
		else {
			if ((10 < cd) && (16 >= cd))
				character = ':' + cd - 11;
			else
				character = '-' + cd - 16;
		}
	}
	[self setCheckDigit:character];
}

- (void)setContent:(NSString *)inContent
// 抜き出しの基本ルールは次のとおりに日本郵政公社が説明している。 //
// 1.まず、データ内にあるアルファベットの小文字は大文字に置き換える。2.同様に、データ内にある'&','/','・','.'は取り除き、後ろのデータを詰める。3.次に算用数字、ハイフン及び連続していないアルファベット１文字を必要な文字情報として抜き出す。4.次に抜き出された文字の前にある「漢字」「カタカナ文字」「漢数字」「プランク」「2文字以上連続したアルファベット文字」は、ハイフン１文字に置き換える。5.ハイフンが連続する場合は1つにまとめる。6.最後に、先頭がハイフンの場合は取り除く。 //
// このあと補足ルールに従って補足する。1.漢数字が'丁目'、'丁'、'番地'、'番'、'号'、'地割'、'線'、'の'、'ノ'の前にある場合は抜き出し対象とし、算用数字に変換して抜き出す。2.連続していないアルファベット１文字は抜き出し対象となるが、算用数字に続くアルファベット１文字'F'に限っては抜き出し対象としない。3.算用数字に続くアルファベット１文字'F'は抜き出し対象としないが、更に'F'以降のデータに抜き出し対象となる文字がある場合、'F'はハイフン１文字に置き換える。4.抜き出し後のバーコードデータについて、アルファベット文字の前後にあるハイフンは取り除く。5.アルファベット文字の前後にあるハイフンを取り除いた結果、2文字以上の連続したアルファベット文字が残った場合、取り除かないでそのままにする。 //
// [273-0102 東3丁目-20-5 郵便・A&cコーポB604号]は、'27301023-20-5B604'となりチェックデジットは'0' //
// [317-0055 6丁目7-14 ABCビル2F201号室]は、'31700556-7-14-2-201'となりチェックデジットは'不明' //
// [650-00469丁目7-6 郵便シティA棟1F1号]は、'65000469-7-6A1-1'となりチェックデジットは'5' //
// [623-00116-7 LプラザB106]は、'62300116-7LB106'となりチェックデジットは'4' //
// ここでは次のように行う。 //
// 大文字化する。20文字に揃える。使用できない文字を排除する。郵便番号の整合性はチェックしない。 //
// 基本ルール3.と補足ルール5.とは互いに矛盾している。従ってここで設定されたcontentを使ってもう一度ここを呼び出すと1回目の結果とは異なった結果になる。 //
{
	unichar uChar,letterBuffer;
	NSUInteger i,jpLength;
	NSString *uppercaseString;
	NSRange range;
	NSMutableString *tempStr;
	NSString *state;
	NSString *tenStr = @"0123456789";
	NSUInteger maxLength = 20; // 郵便番号7文字+住居表示番号13文字 //
	NSMutableString *tempContents = [NSMutableString string];

	for (i = 0; i < [inContent length]; i++) {
		uChar = [inContent characterAtIndex:i];
		if (('&' != uChar) && ('/' != uChar) && ('.' != uChar) && ('\\' != uChar)) // '&','/','.'を取り除く。中グロは取り除けない。localizedはここでやる仕事ではない。中グロがescapeに置き換えられていれば削除できる。 //
			[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
	}
	uppercaseString = [tempContents uppercaseString];
	tempContents = [NSMutableString string];
	for (state = @"other", letterBuffer = '\0', jpLength = 0, i = 0; i < [uppercaseString length]; i++) { // 基本ルールと補足ルール2.3.による抜き出し //
		uChar = [uppercaseString characterAtIndex:i];
		if ((JP_MINIMUMLENGTH < jpLength) && (('A' > uChar) || ('Z' < uChar)) && (YES == [state isEqualToString:@"letter"]) && ('\0' != letterBuffer)) { // 連続していないアルファベット１文字を抜き出す。 //
			[tempContents appendString:[NSString stringWithCharacters:&letterBuffer length:1]];
			jpLength++;
		}
		if ('-' == uChar) { // jpLengthのチェックにより先頭がハイフンになることを防ぐ。characterAtIndexのチェックにより連続することを防ぐ。 //
			if ((NO == [state isEqualToString:@"minus"]) && (JP_MINIMUMLENGTH < jpLength) && ('-' != [tempContents characterAtIndex:jpLength - 1]))
				[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
			state = @"minus";
		}
		else {
			if (('0' <= uChar) && ('9' >= uChar)) {
				[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
				state = @"numeric";
			}
			else {
				if (('A' <= uChar) && ('Z' >= uChar)) {
					if (YES == [state isEqualToString:@"letter"]) { // 連続したアルファベットの2文字目の処理 //
						if ((JP_MINIMUMLENGTH < jpLength) && ('-' != [tempContents characterAtIndex:jpLength - 1])) { // 「2文字以上連続したアルファベット文字」をハイフンに置き換える。 //
							uChar = '-';
							[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
						}
						uChar = '\0'; // 連続したアルファベットであることを示す。 //
					}
					if ((YES == [state isEqualToString:@"numeric"]) && ('F' == uChar)) { // 算用数字に続く'F'の処理 //
						if ((JP_MINIMUMLENGTH < jpLength) && ('-' != [tempContents characterAtIndex:jpLength - 1])) {
							uChar = '-';
							[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
						}
						uChar = '\0';
					}
					state = @"letter";
				}
				else {
					if ((NO == [state isEqualToString:@"other"]) && (JP_MINIMUMLENGTH < jpLength) && ('-' != [tempContents characterAtIndex:jpLength - 1])) { // 「漢字」「カタカナ文字」「漢数字」「プランク」をハイフンに置き換える。末尾が'-'になるときもある。 //
						uChar = '-';
						[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
					}
					state = @"other";
				}
			}
		}
		/* NSLog(@"tempContents=%@",tempContents); */
		jpLength = [tempContents length];
		if (NO == [state isEqualToString:@"letter"])
			letterBuffer = '\0';
		else
			letterBuffer = uChar;
	}
	uppercaseString = [NSString stringWithString:tempContents];
	tempStr = [NSMutableString string];
	tempContents = [NSMutableString string];
	for (state = @"other", jpLength = 0, i = 0; i < [uppercaseString length]; i++) {
		uChar = [uppercaseString characterAtIndex:i];
		if (('A' <= uChar) && ('Z' >= uChar)) {
			if ((YES == [state isEqualToString:@"minus"]) && (JP_MINIMUMLENGTH < jpLength) && ('-' == [tempStr characterAtIndex:jpLength - 1])) { // アルファベット文字の前にあるハイフンを取り除く。 //
				[tempStr deleteCharactersInRange:NSMakeRange(jpLength - 1,1)];
				range = NSMakeRange([tempContents length] - 1,1);
				[tempContents deleteCharactersInRange:range];
			}
			if ('K' > uChar) {
				[tempStr appendString:@":"]; // CC1 //
				[tempStr appendString:[tenStr substringWithRange:NSMakeRange((NSUInteger)(uChar - 'A'),1)]];
				[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
				state = @"letter";
			}
			else {
				if ('U' > uChar) {
					[tempStr appendString:@";"]; // CC2 //
					[tempStr appendString:[tenStr substringWithRange:NSMakeRange((NSUInteger)(uChar - 'K'),1)]];
					[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
					state = @"letter";
				}
				else { // ('Z' >= uChar) //
					[tempStr appendString:@"<"]; // CC3 //
					[tempStr appendString:[tenStr substringWithRange:NSMakeRange((NSUInteger)(uChar - 'U'),1)]];
					[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
					state = @"letter";
				}
			}
		}
		else {
			if ('-' == uChar) {
				if (NO == [state isEqualToString:@"letter"]) { // アルファベット文字の後ろにあるハイフンを取り除く。 //
					[tempStr appendString:[NSString stringWithCharacters:&uChar length:1]];
					[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
				}
				state = @"minus";
			}
			else {
				if (('0' <= uChar) && ('9' >= uChar)) {
					[tempStr appendString:[NSString stringWithCharacters:&uChar length:1]];
					[tempContents appendString:[NSString stringWithCharacters:&uChar length:1]];
					state = @"numeric";
				}
				else
					state = @"other";
			}
		}
		jpLength = [tempStr length];
	}

	if (maxLength < jpLength) { // 21桁目のアルファベットを削除する。その結果、20桁目が制御コードのみになることがある。 //
		[tempStr deleteCharactersInRange:NSMakeRange(maxLength,jpLength - maxLength)];
		jpLength--;
	}
	if ((0 < jpLength) && ('-' == [tempStr characterAtIndex:jpLength - 1])) {
		[tempStr deleteCharactersInRange:NSMakeRange(jpLength - 1,1)];
		range = NSMakeRange([tempContents length] - 1,1);
		[tempContents deleteCharactersInRange:range];
		jpLength--;
	}
	if (maxLength > jpLength) { // パディング //
		for (i = jpLength; i < maxLength; i++) {
			[tempStr appendString:@"="]; // CC4 //
			jpLength++;
		}
	}
	else { // inContentを修正した結果をtempContentsに格納する。 //
		NSString *tempCStr = [NSString stringWithString:tempContents];
		NSUInteger validCount = 0;

		for (i = 0; i < maxLength; i++) {
			uChar = [tempStr characterAtIndex:i];
			if (('-' == uChar) || (('0' <= uChar) && ('9' >= uChar))) // 'A'~'Z'も制御文字+数字文字になっているのでカウントできる。 //
				validCount++;
		}
		tempContents = [NSMutableString stringWithString:[tempCStr substringWithRange:NSMakeRange(0,validCount)]];
	}
	[japanpostContents release];
	japanpostContents = [[NSString alloc] initWithString:tempStr];
    [content release];
    content = [[NSString alloc] initWithString:tempContents];
	[self generateChecksum];
}

- (BOOL)isContentValid
{
	NSUInteger i;
	unichar uChar;
	BOOL result = YES;

	for (i = 0; i < JP_MINIMUMLENGTH; i++) {
		uChar = [japanpostContents characterAtIndex:i];
		if (('0' > uChar) || ('9' < uChar)) {
			result = NO;
			break;
		}
	}
	return result;
}

- (NSString *)barcode
// japanpostContentsを参照するところがNKDBarcodeと異なる。 //
{
	NSUInteger i;
	NSMutableString *theReturn = [NSMutableString string];

	for (i = 0; i < [japanpostContents length]; i++)
		[theReturn appendString:[self _encodeChar:[japanpostContents characterAtIndex:i]]];
	if (checkDigit != -1)
		[theReturn appendString:[self _encodeChar:checkDigit]];
	return theReturn;
}

- (BOOL)printsCaption {
	return NO;
}

- (NSString *)initiator {
	return [barFormat substringToIndex:2 * openBracketLength];
}
- (NSString *)terminator {
	return [barFormat substringToIndex:2 * closeBracketLength - 1]; // 最後の"0"は不要 //
}

- (CGFloat)barTop:(NSInteger)index {
    return (CGFloat)(japanpost_barTop([self _barDescriptor:(NSUInteger)index],(double)height) / (double)MILLIMETERPERPOINT);
}
- (CGFloat)barBottom:(NSInteger)index {
    return (CGFloat)(japanpost_barBottom([self _barDescriptor:(NSUInteger)index],(double)height) / (double)MILLIMETERPERPOINT);    
}

- (NSString *)_encodeChar:(char)inChar {
	return [barFormat substringToIndex:2 * numberLength];
}

- (void)calculateWidth
{
    [self setWidth:(CGFloat)((double)[[self completeBarcode] length] * (double)[self barWidth])];
}

@end
