// -----------------------------------------------------------------------------------
//  NKDBarcode.m
// -----------------------------------------------------------------------------------
//  Created by Jeff LaMarche on Wed May 01 2002.
//  Copyright (c) 2002 Naked Software. All rights reserved.
// -----------------------------------------------------------------------------------
// THIS	SOURCE CODE IS PROVIDED AS-IS WITH NO WARRANTY OF ANY KIND
// -----------------------------------------------------------------------------------
// You may use and redistribute this source code without limitation
// -----------------------------------------------------------------------------------
#import "NKDBarcode.h"


@implementation NKDBarcode

NSString *barcodeTypeStrings = @"BarcodeType";

// -----------------------------------------------------------------------------------
-(id)initWithContent: (NSString *)inContent
// -----------------------------------------------------------------------------------
{
    return [self initWithContent: inContent
                   printsCaption: YES];
}
// -----------------------------------------------------------------------------------
-(id)initWithContent: (NSString *)inContent
       printsCaption: (BOOL)inPrints
// -----------------------------------------------------------------------------------
{
    return [self initWithContent:inContent
                   printsCaption:inPrints
                     andBarWidth:.013*kScreenResolution
                       andHeight:.5*kScreenResolution
                     andFontSize:6.0
                   andCheckDigit:(char)-1];
}
// -----------------------------------------------------------------------------------
-(id)initWithContent: (NSString *)inContent
       printsCaption: (BOOL)inPrints
         andBarWidth: (CGFloat)inBarWidth
           andHeight: (CGFloat)inHeight
         andFontSize: (CGFloat)inFontSize
       andCheckDigit: (char)inDigit
// -----------------------------------------------------------------------------------
{
    /* if (self = [super init]) */ // SatoAkira delete. //
	if (nil != (self = [super init])) // SatoAkira add. //
    {
        if (!inContent)
        {
            [self release];
            return nil;
        }
		[self setCheckDigit: inDigit]; // SatoAkira add. checkDigit will be corrected if required when defining content, after defining checkDigit for the time being. //
        [self setContent:inContent];

        [self setFontSize: inFontSize]; // SatoAkira add.  //
        [self setBarWidth:inBarWidth];
        [self setPrintsCaption:inPrints];

        if (![self printsCaption])
            [self setHeight:inHeight];
        else
            /* [self setHeight:inHeight + (captionHeight*kScreenResolution)]; */
            [self setHeight:inHeight];

        // Calculate width based on number of bars needed to encode this content
        [self calculateWidth];

        /* [self setFontSize: inFontSize]; */ // SatoAkira delete. //

        // Check digit of -1 means no check digit - couldn't use 0 because
        // 0 could be a valid check digit. Typically calculated, not set
        // outside of the class
        /* [self setCheckDigit: inDigit]; */ // SatoAkira delete. //
    }
    return self;
}
// -----------------------------------------------------------------------------------
-(void)setContent:(NSString *)inContent
// -----------------------------------------------------------------------------------
{
    /* [content autorelease];
    content = [inContent retain]; */ // SatoAkira delete. //
    [content release]; // SatoAkira add. //
    content = [[NSString alloc] initWithString:inContent]; // SatoAkira add. //
	[self generateChecksum]; // SatoAkira add. //
}
// -----------------------------------------------------------------------------------
-(void)setHeight:(CGFloat)inHeight
// -----------------------------------------------------------------------------------
{
    height = inHeight;
}
// -----------------------------------------------------------------------------------
-(void)setWidth:(CGFloat)inWidth
// -----------------------------------------------------------------------------------
{
    width = inWidth;
}
// -----------------------------------------------------------------------------------
-(void)calculateWidth
// -----------------------------------------------------------------------------------
{
    [self setWidth:[[self completeBarcode] length] * [self barWidth]];
}
// -----------------------------------------------------------------------------------
-(void)setCheckDigit:(char)inCheckDigit
// -----------------------------------------------------------------------------------
{
    checkDigit = inCheckDigit;
}
// -----------------------------------------------------------------------------------
-(void)setPrintsCaption:(BOOL)inPrints
// -----------------------------------------------------------------------------------
{
    printsCaption = inPrints;
}
// -----------------------------------------------------------------------------------
-(void)setBarWidth:(CGFloat)inBarWidth
// -----------------------------------------------------------------------------------
{
    barWidth = inBarWidth;
}
// -----------------------------------------------------------------------------------
-(NSString *)content
// -----------------------------------------------------------------------------------
{
    return content;
}
// -----------------------------------------------------------------------------------
-(CGFloat)height
// -----------------------------------------------------------------------------------
{
    return height;
}
// -----------------------------------------------------------------------------------
-(CGFloat)width
// -----------------------------------------------------------------------------------
{
    return width;
}
// -----------------------------------------------------------------------------------
-(char)checkDigit
// -----------------------------------------------------------------------------------
{
    return checkDigit;
}
// -----------------------------------------------------------------------------------
-(BOOL)printsCaption
// -----------------------------------------------------------------------------------
{
    return printsCaption;
}
// -----------------------------------------------------------------------------------
-(CGFloat)barWidth
// -----------------------------------------------------------------------------------
{
    return barWidth;
}
// -----------------------------------------------------------------------------------
-(BOOL)isSizeValid
// -----------------------------------------------------------------------------------
{
    return YES;
}
// -----------------------------------------------------------------------------------
-(void)generateChecksum
// -----------------------------------------------------------------------------------
{
    // Stub
}
// -----------------------------------------------------------------------------------
-(BOOL) isContentValid
// -----------------------------------------------------------------------------------
{
    NSInteger			i;
    char		*contentString;

    /* contentString = (char *)[[self content] lossyCString]; */
    contentString = (char *)[[self content] UTF8String];

    for (i = 0; i < strlen(contentString); i++)
        if ([[self _encodeChar:contentString[i]] isEqual:@""])
            return NO;
	return YES;
}
// -----------------------------------------------------------------------------------
-(NSString *)initiator
// -----------------------------------------------------------------------------------
{
    return @"";
}
// -----------------------------------------------------------------------------------
-(NSString *)terminator
// -----------------------------------------------------------------------------------
{
    return @"";
}
// -----------------------------------------------------------------------------------
-(NSString *)_encodeChar:(char)inChar
// -----------------------------------------------------------------------------------
{
    // Subclasses MUST override - we'll encode dummy character for testing
    return @"1010";
}
// -----------------------------------------------------------------------------------
-(NSString *)barcode
// -----------------------------------------------------------------------------------
{
    NSMutableString 	*theReturn = [NSMutableString stringWithString:@""];
    NSInteger			i;
    char		*contentString;

    /* contentString = (char *)[[self content] lossyCString]; */
    contentString = (char *)[[self content] UTF8String];

    for (i = 0; i < strlen(contentString); i++)
        [theReturn appendString:[self _encodeChar:contentString[i]]];

    if (checkDigit != -1)
        [theReturn appendString:[self _encodeChar:checkDigit]];

    return theReturn;

}
// -----------------------------------------------------------------------------------
-(NSString *)completeBarcode
// -----------------------------------------------------------------------------------
{
    return  [NSString stringWithFormat:@"%@%@%@", [self initiator], [self barcode], [self terminator]];
}
// -----------------------------------------------------------------------------------
-(void)setCaptionHeight:(CGFloat)inHeight
// -----------------------------------------------------------------------------------
{
    captionHeight = inHeight;
}
// -----------------------------------------------------------------------------------
-(CGFloat)captionHeight
// -----------------------------------------------------------------------------------
{
    return captionHeight;
}
// -----------------------------------------------------------------------------------
-(void)setFontSize:(CGFloat)inSize
// -----------------------------------------------------------------------------------
{
    NSFont *font;
    
    fontSize = inSize;

	font = [NSFont userFixedPitchFontOfSize:[self fontSize]];
    //font = [NSFont fontWithName:@"Geneva" size:[self fontSize]];
	
    [self setCaptionHeight: ([font capHeight]/72) + (.35 * ([font capHeight]/72))];  
}
// -----------------------------------------------------------------------------------
-(CGFloat) fontSize
// -----------------------------------------------------------------------------------
{
    return fontSize;
}
// -----------------------------------------------------------------------------------
-(NSInteger)digitsToLeft
// -----------------------------------------------------------------------------------
{
    return 0;
}
// -----------------------------------------------------------------------------------
-(NSInteger)digitsToRight
// -----------------------------------------------------------------------------------
{
    return 0;
}
// -----------------------------------------------------------------------------------
-(NSString *)leftCaption
// -----------------------------------------------------------------------------------
{
    return [content substringWithRange:NSMakeRange(0, [self digitsToLeft])];
}
// -----------------------------------------------------------------------------------
-(NSString *)rightCaption
// -----------------------------------------------------------------------------------
{
    return [content substringWithRange:NSMakeRange([content length] - [self digitsToRight], [self digitsToRight])];
}
// -----------------------------------------------------------------------------------
-(NSString *)caption
// -----------------------------------------------------------------------------------
{
    if ( ([self digitsToRight] == 0) && ([self digitsToLeft] == 0))
        return content;
    else
        return [content substringWithRange:NSMakeRange([self digitsToLeft],
                                                       [content length] - [self digitsToLeft] - [self digitsToRight])];
}
// -----------------------------------------------------------------------------------
-(CGFloat)barBottom:(NSInteger)index
// -----------------------------------------------------------------------------------
{
    if ([self printsCaption])
        return  captionHeight * kScreenResolution;
    else
        return 0.0;
}
// -----------------------------------------------------------------------------------
-(CGFloat)barTop:(NSInteger)index
// -----------------------------------------------------------------------------------
{
        return [self height];
}
// -----------------------------------------------------------------------------------
-(CGFloat)firstBar
// -----------------------------------------------------------------------------------
{
    return 0;
}
// -----------------------------------------------------------------------------------
-(CGFloat)lastBar
// -----------------------------------------------------------------------------------
{
    return [self width] - [self barWidth];
}
// -----------------------------------------------------------------------------------
-(NSString *)description
// -----------------------------------------------------------------------------------
{
    return [NSString stringWithFormat:@"\n\tBarcode Class: %@\n\tContent: %@\n\tCheck Digit:%c\n\tWidth:%f\n\tHeight:%f\n\tBar Width:%f\n\tFont Size:%f\n\tCaption Height:%f",
        [self class],
        [self content],
        [self checkDigit],
        [self width],
        [self height],
        [self barWidth],
        [self fontSize],
        [self captionHeight]];
}
// -----------------------------------------------------------------------------------
- (BOOL)isEqual:(id)anObject
// -----------------------------------------------------------------------------------
{
    NKDBarcode	*compare;

    compare = anObject;

    return  ([[self content] isEqual:[compare content]]);
}
// -----------------------------------------------------------------------------------
- (id)copyWithZone:(NSZone *)zone
// -----------------------------------------------------------------------------------
{
	CGFloat selfHeight = (YES == [self printsCaption]) ? [self height] - [self captionHeight] * kScreenResolution : [self height]; // SatoAkira add. //
    NKDBarcode *ret =  [[[self class] alloc] initWithContent: [self content]
                                   printsCaption: [self printsCaption]
                                     andBarWidth: [self barWidth]
                                       /* andHeight: [self height] */ // SatoAkira delete. //
									   andHeight:selfHeight
                                     andFontSize: [self fontSize]
                                   andCheckDigit: [self checkDigit]];
    [ret calculateWidth];
    return ret;
}
// -----------------------------------------------------------------------------------
- (id)initWithCoder:(NSCoder *)coder
// -----------------------------------------------------------------------------------
{


    // Since we subclass NSObject, this call to super
    // init is not necessary, but it's good form to
    // include it, as it is possible that someday
    // NSObject's -init method will do something
    // -----------------------------------------------
    self = 	[super init];

    [self setContent:[coder decodeObject]];
    [coder decodeValueOfObjCType:@encode(BOOL)  at:&printsCaption];
    [coder decodeValueOfObjCType:@encode(CGFloat) at:&barWidth];
    [coder decodeValueOfObjCType:@encode(CGFloat) at:&width];
    [coder decodeValueOfObjCType:@encode(CGFloat) at:&height];
    [coder decodeValueOfObjCType:@encode(CGFloat) at:&fontSize];
    [coder decodeValueOfObjCType:@encode(CGFloat) at:&captionHeight];
    [coder decodeValueOfObjCType:@encode(char)  at:&checkDigit];

    return self;
}
//----------------------------------------------------------------------
- (void) encodeWithCoder: (NSCoder *)coder
//----------------------------------------------------------------------
{
    [coder encodeObject:[self content]];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&printsCaption];
    [coder encodeValueOfObjCType:@encode(CGFloat) at:&barWidth];
    [coder encodeValueOfObjCType:@encode(CGFloat) at:&width];
    [coder encodeValueOfObjCType:@encode(CGFloat) at:&height];
    [coder encodeValueOfObjCType:@encode(CGFloat) at:&fontSize];
    [coder encodeValueOfObjCType:@encode(CGFloat) at:&captionHeight];
    [coder encodeValueOfObjCType:@encode(char) at:&checkDigit];

}
// -----------------------------------------------------------------------------------
-(void)dealloc
// -----------------------------------------------------------------------------------
{
    [content release];
    [super dealloc];
}

@end
