//
//  ASSoundController.m
//
//  Created by 佐藤 昭 on Thu Jul 18 2002.
//  Copyright (c) 2002 SatoAkira. All rights reserved.
//

#import "ASSoundController.h"


@implementation ASSoundController

- (void)setReleasedWhenClosed:(BOOL)flag {
	releasedWhenClosed = flag;
}
+ (ASSoundController *)soundController
{
	ASSoundController *newInstance = [[self alloc] init];
	[newInstance setReleasedWhenClosed:YES]; // ファクトリーメソッドから作る場合にはYESにする。 //
	return newInstance;
}
+ (ASSoundController *)sharedSoundController
{
	static ASSoundController *sharedSoundController = nil;

	if (nil == sharedSoundController)
		sharedSoundController = [[self alloc] init];
	return sharedSoundController;
}
- (id)initWithWindowNibName:(NSString *)windowNibName owner:(id)owner {
	if (nil != (self = [super initWithWindowNibName:windowNibName owner:owner])) {
		releasedWhenClosed = NO;
		soundData = nil;
		theSound = nil;
		url = nil;
	}
	return self;
}
- (id)initWithWindowNibName:(NSString *)windowNibName {
	return [self initWithWindowNibName:windowNibName owner:self];
}
- (id)init {
	return [self initWithWindowNibName:NSStringFromClass([self class]) owner:self];
}
- (void)dealloc
{
	[[self window] setDelegate:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[NSNotificationCenter defaultCenter] postNotificationName:NSWindowWillCloseNotification object:self];
	[soundData release];
	[theSound release];
	[url release];
	[super dealloc];
}

- (void)windowDidLoad
{
	[playTitleField setStringValue:NSLocalizedStringFromTable(@"Play",NSStringFromClass([self class]),@"")];
	[stopTitleField setStringValue:NSLocalizedStringFromTable(@"Stop",NSStringFromClass([self class]),@"")];
	[pauseTitleField setStringValue:NSLocalizedStringFromTable(@"Pause",NSStringFromClass([self class]),@"")];
	[playButton setToolTip:NSLocalizedStringFromTable(@"Play",NSStringFromClass([self class]),@"")];
	[stopButton setToolTip:NSLocalizedStringFromTable(@"Stop",NSStringFromClass([self class]),@"")];
	[pauseButton setToolTip:NSLocalizedStringFromTable(@"Pause",NSStringFromClass([self class]),@"")];
	[openFileButton setTitle:NSLocalizedStringFromTable(@"Open Sound File...",NSStringFromClass([self class]),@"")];
	[playButton setEnabled:NO];
	[stopButton setEnabled:NO];
	[pauseButton setEnabled:NO];
    [soundFileWell setImage:[NSImage imageNamed:@"SoundFileWellIcon"]];
	[soundFileWell setAction:@selector(draggingDidEnd:)];
	[soundFileWell setTarget:self];
	[soundFileWell setToolTip:NSLocalizedStringFromTable(@"Drag fileicon of sound here.",NSStringFromClass([self class]),@"")];
	[[self window] setDelegate:self];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:[[self document] windowController]];
}
- (void)windowWillClose:(NSNotification *)aNotification {
	[[self window] orderOut:aNotification]; // これが重要。これをやらないと[self window]が消え残ってしまう。 //
	if (YES == releasedWhenClosed)
		[self release];
	else {
		if ([aNotification object] == [[self document] windowController])
			;
		// else 単純にクローズボタンがクリックされただけ
	}
}
- (BOOL)setSoundData:(NSData *)aSound
{
	if (nil != theSound)
		[theSound setDelegate:nil];
	[soundData release];
	[theSound release];
	if ((nil != aSound) && (0 < [aSound length])) {
		NSSound *newSound;

		if (nil != (newSound = [[NSSound alloc] initWithData:aSound])) {
			soundData = [[NSData alloc] initWithData:aSound];
			theSound = newSound;
			[theSound setDelegate:self];
			[playButton setEnabled:YES];
			[stopButton setEnabled:NO];
			[pauseButton setEnabled:NO];
			return YES;
		}
		else {
			soundData = nil;
			theSound = nil;
			[playButton setEnabled:NO];
			[stopButton setEnabled:NO];
			[pauseButton setEnabled:NO];
            NSAlert *alert = [[NSAlert alloc] init];
            // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
            [alert setMessageText:NSLocalizedStringFromTable(@"Error!",NSStringFromClass([self class]),@"エラー発生!")];
            [alert setInformativeText:NSLocalizedStringFromTable(@"Data is bad.",NSStringFromClass([self class]),@"")];
            [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
            [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
                // 何もやることがない //
            }];
            [alert release];
			/* NSBeginAlertSheet(NSLocalizedStringFromTable(@"Error!",NSStringFromClass([self class]),@"エラー発生!"),nil,nil,nil,[self window],self,NULL,NULL,NULL,NSLocalizedStringFromTable(@"Data is bad.",NSStringFromClass([self class]),@""),nil); */
			return NO;
		}
	}
	else {
		soundData = nil;
		theSound = nil;
		[playButton setEnabled:NO];
		[stopButton setEnabled:NO];
		[pauseButton setEnabled:NO];
		return YES;
	}
}
- (NSData *)soundData {
	return soundData;
}

- (void)setURLWithPath:(NSString *)path {
	[url release];
	url = (nil != path) ? [[NSURL alloc] initFileURLWithPath:path] : nil;
}
- (void)setURL:(NSURL *)aValue {
	[url release];
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_11
    url = (nil != aValue) ? [[NSURL alloc] initFileURLWithPath:[aValue path]] : nil;
#else
	url = (nil != aValue) ? [[NSURL alloc] initWithScheme:[aValue scheme] host:[aValue host] path:[aValue path]] : nil;
#endif
}
- (NSURL *)url {
	return url;
}

- (void)setSoundDataWithContentsOfURL:(NSURL *)aURL
{
	if (nil != aURL) {
		if (YES == [self setSoundData:[NSData dataWithContentsOfURL:aURL]])
			[self setURL:aURL];
		else
			[self setURL:nil];
	}
	else {
		[self setSoundData:nil];
		[self setURL:nil];
	}
	[soundFileWell setURL:nil]; // dragではないのでnilとする。 //
}
- (void)soundDataDidChange:(NSNotification *)notification
{
	NSDictionary *info = [notification userInfo];

	if (nil != info) {
		NSURL *aURL = [info objectForKey:ASSoundControllerURLKey];

		if (nil != aURL)
			[self setSoundDataWithContentsOfURL:aURL];
	}
	else {
		[self setSoundData:nil];
		[self setURL:nil];
	}
}

- (IBAction)openSoundFile:(id)sender
{
	NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setDirectoryURL:nil];
	[oPanel setNameFieldStringValue:@""];
	[oPanel setCanChooseDirectories:NO];
	[oPanel setAllowedFileTypes:[[SoundFileWell class] availableTypes]];
	[oPanel setTitle:@""]; // nilでは駄目。 //
	[oPanel setMessage:@""];
	[oPanel setAllowsMultipleSelection:NO]; // 複雑になるので単一とする。 //
	[oPanel beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result)
	 {
		 if (NSModalResponseOK == result) {
			 NSURL *aURL = [[oPanel URLs] lastObject];
			 if (YES == [self setSoundData:[NSData dataWithContentsOfURL:aURL]])
				 [self setURL:aURL];
			 else
				 [self setURL:nil];
			 [soundFileWell setURL:nil]; // dragではないのでnilとする。 //
		 }
	 }
	 ];
}

- (IBAction)draggingDidEnd:(id)sender
{
	NSURL *aURL = [sender url];

	if (YES == [self setSoundData:[NSData dataWithContentsOfURL:aURL]])
		[self setURL:aURL];
	else
		[self setURL:nil];
}

- (IBAction)playSound:(id)sender
{
	[sender setEnabled:NO];
	[stopButton setEnabled:YES];
	[pauseButton setEnabled:YES];
	[theSound play]; // aiffのみ再生可能 //
}
- (IBAction)stopSound:(id)sender
{
	if (NSControlStateValueOn == [pauseButton state]) {
		[pauseButton setState:NSControlStateValueOff];
		[theSound resume];
	}
	[theSound stop];
}
- (IBAction)pauseSound:(id)sender
{
	if (NSControlStateValueOn == [sender state]) {
		[theSound pause];
	}
	else {
		[theSound resume];
	}
}
- (void)sound:(NSSound *)sound didFinishPlaying:(BOOL)aBool
{
	[playButton setState:NSControlStateValueOff];
	[playButton setEnabled:YES];
	[stopButton setEnabled:NO];
	[pauseButton setEnabled:NO];
}
- (void)hadError:(id)sender
{
    NSAlert *alert = [[NSAlert alloc] init];
    // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
    [alert setMessageText:NSLocalizedStringFromTable(@"Error!",NSStringFromClass([self class]),@"エラー発生!")];
    [alert setInformativeText:NSLocalizedStringFromTable(@"Data is bad.",NSStringFromClass([self class]),@"")];
    [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
    [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
        // 何もやることがない //
    }];
    [alert release];
	/* NSBeginAlertSheet(NSLocalizedStringFromTable(@"Error!",NSStringFromClass([self class]),@"エラー発生!"),nil,nil,nil,[self window],self,NULL,NULL,NULL,NSLocalizedStringFromTable(@"Data is bad.",NSStringFromClass([self class]),@""),nil); */
	[playButton setState:NSControlStateValueOff];
	[playButton setEnabled:YES];
	[stopButton setEnabled:NO];
	[pauseButton setEnabled:NO];
}

@end
