//
//  ASBarcodeView.m
//  BarcodePod
//
//  Created by 佐藤 昭 on Tue Jun 03 2003.
//  Copyright (c) 2003 SatoAkira. All rights reserved.
//

#import "HagakiBarcodeView.h"

@implementation HagakiBarcodeView

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
		barcode = nil;
		scaling = NO;
    }
    return self;
}

- (NSRect)barcodeBounds
{
	NSString *codeString = (nil != barcode) ? [barcode completeBarcode] : nil;

	if ((nil != barcode) && (nil != codeString) && (NO == [codeString isEqualToString:@""]) && (YES == [barcode isContentValid])) {
		CGFloat heightSpace = 5.669303; // 2mm //
		CGFloat widthSpace = 10.0 * [barcode barWidth];

		return NSMakeRect(-0.5 * [barcode width] - widthSpace,-0.5 * [barcode height] - heightSpace,[barcode width] + 2.0 * widthSpace,[barcode height] + 2.0 * heightSpace);
	}
	else
		return NSZeroRect;
}

- (void)drawRect:(NSRect)rect
{
	NSRect bounds = [self bounds];
	NSRect bBounds = [self barcodeBounds];
	NSAffineTransform *transform = [NSAffineTransform transform];
	NSPoint center = NSMakePoint(NSMidX(bounds),NSMidY(bounds));
	NSGraphicsContext *currentContext = [NSGraphicsContext currentContext];
	NSSize borderSize = NSMakeSize(6.0,6.0);

    [transform translateXBy:center.x yBy:center.y];
	if ((nil != barcode) && (NO == NSEqualRects(NSZeroRect,bBounds))) {
		NSUInteger i;
		NSInteger barCount;
		CGFloat scaleX,scaleY;
		/* NSBezierPath *path; */
		NSString *codeString = [barcode completeBarcode];
		CGFloat barWidth = [barcode barWidth];
		CGFloat widthSpace = 0.5 * (bBounds.size.width - [barcode width]);
		CGFloat heightSpace = 0.5 * (bBounds.size.height - [barcode height]);
		NSPoint curPos = NSMakePoint(bBounds.origin.x + [barcode firstBar] + widthSpace,bBounds.origin.y + heightSpace);
		BOOL started = NO;
		NSInteger lastBarIndex = -1;

		[currentContext setShouldAntialias:YES];
		if ((YES == scaling) && (YES == [currentContext isDrawingToScreen])) {
			scaleX = (bounds.size.width  - 2.0 * borderSize.width) / bBounds.size.width;
			scaleY = (bounds.size.height  - 2.0 * borderSize.height) / bBounds.size.height;
			if (scaleX > scaleY) // proportionally //
				scaleX = scaleY;
			else
				scaleY = scaleX;
			[transform scaleXBy:scaleX yBy:scaleY];
		}
		// else { scaleX = 1.0; scaleY = 1.0; } //
		[transform concat];
		[[NSColor whiteColor] set];
		NSRectFill(bBounds);
		[[NSColor blackColor] set];       
		for (barCount = 0, i = 0; i < [codeString length]; i++) {
			if ('1' == [codeString characterAtIndex:i]) {
				if (NO == started)
					started = YES;
				barCount++;
				lastBarIndex = (int)i;

				// If last character is a bar, it needs to be printed here.
				if ([codeString length] - 1 == i)
					NSRectFill(NSMakeRect(curPos.x,curPos.y + [barcode barBottom:lastBarIndex],barWidth * barCount,[barcode barTop:lastBarIndex] - [barcode barBottom:lastBarIndex]));
			}
			else {
				if (YES == started)
					NSRectFill(NSMakeRect(curPos.x,curPos.y + [barcode barBottom:lastBarIndex],barWidth * barCount,[barcode barTop:lastBarIndex] - [barcode barBottom:lastBarIndex]));
				curPos.x += barWidth * (barCount + 1);
				barCount = 0;
				started = NO;
			}
		}
	}
	else {
		[transform concat];
		[[NSColor whiteColor] set];
		NSRectFill(NSMakeRect(-0.5 * bounds.size.width,-0.5 * bounds.size.height,bounds.size.width,bounds.size.height));
	}
}

- (void)setBarcode:(NKDBarcode *)anObject
{
	if (nil != anObject) {
		[barcode release];
		barcode = [anObject copyWithZone:[self zone]];
	}
	else {
		[barcode release];
		barcode = nil;
	}
}
- (NKDBarcode *)barcode {
	return barcode;
}

- (void)setScaling:(BOOL)flag {
	scaling = flag;
}
- (BOOL)scaling {
	return scaling;
}

@end
