/* HagakiPrint.m created by SatoAkira on Mon 19-Jul-1999 */

#import "HagakiPrint.h"
#import "MainDocument.h"
#import "EnrollFunctions.h" /* for strntrim() */

@implementation HagakiPrint

- (void)setReleasedWhenClosed:(BOOL)flag {
	releasedWhenClosed = flag;
}
+ (HagakiPrint *)hagakiPrint {
	HagakiPrint *newInstance = [[self alloc] init];
	[newInstance setReleasedWhenClosed:YES];  // ファクトリーメソッドから作る場合にはYESにする。 //
	return newInstance;
}

- (instancetype)init {
	if (nil != (self = [super initWithWindowNibName:NSStringFromClass([self class])])) {
		NSSize postcardSize = NSMakeSize(419.52,283.46);

		misterString = nil;
		releasedWhenClosed = NO;
		printInfo = [[NSPrintInfo alloc] initWithDictionary:[[NSPrintInfo sharedPrintInfo] dictionary]];
		[printInfo setPaperSize:postcardSize]; // これでpaperNameが「はがき」になる。プリンタ名もついでに設定される。setPaperName:をやると設定が壊される。 //
		[printInfo setOrientation:NSPaperOrientationLandscape];
		[printInfo setLeftMargin:0.0]; // 縁なしと縁ありとでは左上の印字位置が違うはずなのでこのままでは駄目なはずである。 //
		[printInfo setRightMargin:0.0];
		[printInfo setTopMargin:0.0];
		[printInfo setBottomMargin:0.0];
		[printInfo setVerticallyCentered:YES];
		[printInfo setHorizontallyCentered:YES];
		[printInfo setVerticalPagination:NSFitPagination];
		[printInfo setHorizontalPagination:NSFitPagination];
		if (NO == NSEqualSizes([printInfo paperSize],postcardSize)) {
			float deltax = postcardSize.width - [printInfo paperSize].width;
			float deltay = postcardSize.height - [printInfo paperSize].height;

			papersetSuccess = ((1.0 > fabs(deltax)) && (1.0 > fabs(deltay))) ? YES : NO;
		}
		else
			papersetSuccess = YES;
	}
	return self;
}

- (void)dealloc {
	[[self window] setDelegate:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[self window] setDelegate:nil];
	[[NSNotificationCenter defaultCenter] postNotificationName:NSWindowWillCloseNotification object:self];
	[misterString release];
	[printInfo release];
	[super dealloc];
}

- (void)windowDidLoad
{
	NSArray *honorifics = [NSLocalizedString(@"Honorifics",@"") componentsSeparatedByString:@","];
	NSFont *zipFont = [NSFont fontWithName:NSLocalizedString(@"ZipFont",@"") size:14.0];
	NSFont *addressFont = [NSFont fontWithName:NSLocalizedString(@"HagakiAddressFont",@"") size:16.0];
	NSFont *nameFont = [NSFont fontWithName:NSLocalizedString(@"HagakiNameFont",@"") size:18.0];

	if (nil == zipFont)
		zipFont = [NSFont userFixedPitchFontOfSize:14.0];
	[zipTextView  setFont:zipFont];
	if (nil == addressFont)
		addressFont = [NSFont userFixedPitchFontOfSize:14.0];
	[addressTextView setFont:addressFont];
	if (nil == nameFont)
		nameFont = [NSFont boldSystemFontOfSize:16.0];
	
	[nameTextView setFont:nameFont];
	[addressTextView setSelectable:YES];
	[addressTextView setEditable:YES];
	[addressTextView setImportsGraphics:YES];
	[addressTextView setVerticallyResizable:NO];
	[addressTextView setToolTip:NSLocalizedString(@"AddressMessage",@"")];
	[nameTextView setSelectable:YES];
	[nameTextView setEditable:YES];
	[nameTextView setImportsGraphics:YES];
	[nameTextView setVerticallyResizable:NO];
	[nameTextView setToolTip:NSLocalizedString(@"NameMessage",@"")];
	[misterPopUp removeAllItems];
	[misterPopUp addItemsWithTitles:honorifics];
	[misterPopUp addItemWithTitle:@""];
	[misterPopUp selectItemAtIndex:0];
	[misterString release];
	misterString = [[NSString alloc] initWithString:[misterPopUp titleOfSelectedItem]];
	[printButton setTitle:NSLocalizedString(@"Print...",@"")];
	[pageSetupButton setTitle:NSLocalizedString(@"Page Setup...",@"")];
	[misterPositionSwitch setAlternateTitle:NSLocalizedString(@"SufHonorific",@"後敬称")];
	[misterPositionSwitch setTitle:NSLocalizedString(@"PreHonorific",@"前敬称")];
	[stampField setStringValue:NSLocalizedString(@"Stamp",@"消印・切手")];
	[[barcodeContentForm titleAtIndex:0] setStringValue:NSLocalizedString(@"DataOfBarcode",@"")];
	[[barcodeContentForm cellAtIndex:0] setStringValue:@""];
	[barcodeContentForm calcSize];
	[countryField setStringValue:@""];
	[self setWindowFrameAutosaveName:NSStringFromClass([self class])];
	[[self window] setDelegate:self];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:[[self document] windowController]];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(indexDidChange:) name:IndexDidChangeNotification object:[[self document] windowController]];
}

- (void)windowWillClose:(NSNotification *)aNotification {
	[[self window] orderOut:aNotification];
	if (YES == releasedWhenClosed)
		[self release];
}

- (void)didEndPrinting {
	hagakiBox.contentView.appearance = nil; // nilで元に戻る。 //
}
- (IBAction)printDocument:(id)sender {
	if (NO == papersetSuccess)
		[self runPageLayout:sender];
	else {
		hagakiBox.contentView.appearance = [NSAppearance appearanceNamed:NSAppearanceNameAqua];
		NSPrintOperation *op = [NSPrintOperation printOperationWithView:hagakiBox.contentView printInfo:printInfo];
		[op runOperationModalForWindow:self.window delegate:self didRunSelector:@selector(didEndPrinting) contextInfo:NULL];
	}
}
- (void)setBarcode:(NSString *)aString zip:(NSString *)code country:(NSString *)country
{
		NSUInteger i;
		unichar uChar;
		NSString *zipcode,*blockcode;
		NSMutableString *tempStr = [NSMutableString string];
		NSString *countryTag = [countryCodeDic() objectForKey:country];

		if (YES == [countryTag isEqualToString:@"ja"]) { // ja //
			JapanPostBarcode *japanPostBarcode;

			zipcode = (nil == code) ? aString : code;
			blockcode = (nil == code) ? @"" : aString;
			japanPostBarcode = [[JapanPostBarcode alloc] initWithContent:[zipcode stringByAppendingString:blockcode] printsCaption:NO andBarWidth:0.0 andHeight:10.0 andFontSize:0.0 andCheckDigit:' '];
			if ((nil != japanPostBarcode) && (YES == [japanPostBarcode isContentValid])) {
				[[barcodeContentForm cellAtIndex:0] setStringValue:[japanPostBarcode content]];
				[barcodeView setBarcode:japanPostBarcode];
			}
			else {
				[[barcodeContentForm cellAtIndex:0] setStringValue:@""];
				[barcodeView setBarcode:nil];
			}
			[japanPostBarcode release];
		}
		else {
			if (YES == [countryTag isEqualToString:@"us"]) { // us //
				NSMutableString *tempBlock = [NSMutableString string];

				zipcode = (nil == code) ? aString : code;
				for (i = 0; i < [zipcode length]; i++) {
					uChar = [zipcode characterAtIndex:i];
					if (('0' <= uChar) && ('9' >= uChar))
						[tempStr appendString:[NSString stringWithCharacters:&uChar length:1]];
					if (5 <= [tempStr length])
						break;
				}
				if (nil == code)
					blockcode = ([tempStr length] < [aString length]) ? [aString substringFromIndex:[tempStr length]] : @"";
				else
					blockcode = aString;
				for (i = 0; i < [blockcode length]; i++) {
					uChar = [blockcode characterAtIndex:i];
					if (('0' <= uChar) && ('9' >= uChar))
						[tempBlock appendString:[NSString stringWithCharacters:&uChar length:1]];
					if (6 <= [tempBlock length])
						break;
				}
				if (5 == [tempStr length]) {
					NKDPostnetBarcode *postnetBarcode = [[NKDPostnetBarcode alloc] initWithContent:[tempStr stringByAppendingString:tempBlock] printsCaption:NO andBarWidth:1.08 andHeight:9.0 andFontSize:0.0 andCheckDigit:' '];

					if ((nil != postnetBarcode) && (YES == [postnetBarcode isContentValid])) {
						[[barcodeContentForm cellAtIndex:0] setStringValue:[postnetBarcode content]];
						[barcodeView setBarcode:postnetBarcode];
					}
					else {
						[[barcodeContentForm cellAtIndex:0] setStringValue:@""];
						[barcodeView setBarcode:nil];
					}
					[postnetBarcode release];
				}
				else
					[barcodeView setBarcode:nil];
			}
			else {
				if (YES == [countryTag isEqualToString:@"uk"]) { // uk //
					if (nil == code)
						[tempStr appendString:aString];
					else
						[tempStr appendString:code];
					if (nil == code)
						blockcode = @"";
					else
						blockcode = aString;
					if (0 < [tempStr length]) {
						NKDRoyalMailBarcode *royalMailBarcode = [[NKDRoyalMailBarcode alloc] initWithContent:[tempStr stringByAppendingString:blockcode] printsCaption:NO andBarWidth:1.417322835 andHeight:14.31496063 andFontSize:0.0 andCheckDigit:' '];

						if ((nil != royalMailBarcode) && (YES == [royalMailBarcode isContentValid])) {
							[[barcodeContentForm cellAtIndex:0] setStringValue:[royalMailBarcode content]];
							[barcodeView setBarcode:royalMailBarcode];
						}
						else {
							[[barcodeContentForm cellAtIndex:0] setStringValue:@""];
							[barcodeView setBarcode:nil];
						}
						[royalMailBarcode release];
					}
					else
						[barcodeView setBarcode:nil];
				}
				else
					[barcodeView setBarcode:nil];
			}
		}
		[barcodeView setNeedsDisplay:YES];
}

- (void)setAddress:(NSString *)aString zip:(NSString *)code country:(NSString *)country
{
	if (nil != addressTextView) {
		NSUInteger i;
		NSMutableString *zipString = [NSMutableString string];

		for (i = 0; i < [code length]; i++)
			[zipString appendString:NSLocalizedStringFromTable([code substringWithRange:NSMakeRange(i,1)],@"Numeric",@"")];
		[zipTextView setString:zipString];
		[addressTextView setString:aString];
	}
}
- (void)setName:(NSString *)aString
{
	if (nil != nameTextView) {
		if (NSControlStateValueOff == [misterPositionSwitch state])
			[nameTextView setString:[NSString stringWithFormat:@"%@  %@",aString,misterString]];
		else
			[nameTextView setString:[NSString stringWithFormat:@"%@  %@",misterString,aString]];
	}
}
- (void)setMisterString:(id)sender
{
	NSString *vString = [nameTextView string];

	if (NO == [misterString isEqualToString:@""]) {
		NSRange range = [vString rangeOfString:misterString options:(NSControlStateValueOff == [misterPositionSwitch state]) ? NSBackwardsSearch : NSLiteralSearch];
	
		if (0 < range.length) {
			[nameTextView replaceCharactersInRange:range withString:[sender titleOfSelectedItem]];
			[nameTextView setNeedsDisplay:YES];
		}
	}
	else {
		if (NSControlStateValueOff == [misterPositionSwitch state])
			[nameTextView setString:[vString stringByAppendingString:[sender titleOfSelectedItem]]];
		else
			[nameTextView setString:[[sender titleOfSelectedItem] stringByAppendingString:vString]];
		[nameTextView setNeedsDisplay:YES];
	}
	[misterString release];
	misterString = [[NSString alloc] initWithString:[sender titleOfSelectedItem]];
}
- (IBAction)setMisterPosition:(id)sender
{
	NSString *vString = [nameTextView string];

	if (NO == [vString isEqualToString:@""]) {
		NSString *newStr;
		NSRange range = [vString rangeOfString:misterString options:(NSControlStateValueOff == [sender state]) ? NSLiteralSearch : NSBackwardsSearch];

		if (NSControlStateValueOn == [sender state]) {
			newStr = strntrim([vString substringWithRange:NSMakeRange(0,range.location)],0,NO);
			[nameTextView setString:[NSString stringWithFormat:@"%@  %@",misterString,newStr]];
		}
		else {
			newStr = strntrim([vString substringWithRange:NSMakeRange(range.length,[vString length] - range.length)],0,NO);
			[nameTextView setString:[NSString stringWithFormat:@"%@  %@",newStr,misterString]];
		}
		[nameTextView setNeedsDisplay:YES];
	}
}

- (IBAction)setBarcodeContent:(id)sender
{
	[self setBarcode:[[sender cellAtIndex:0] stringValue] zip:nil country:[countryField stringValue]];
}

- (void)runPageLayout:(id)sender
{
	NSPrintInfo *tempPrintInfo = [printInfo copy];
    NSPageLayout *pageLayout = [NSPageLayout pageLayout];
	ASPageLayoutAccessory *accessory = [[ASPageLayoutAccessory alloc] initWithNibName:PageLayoutNibName bundle:nil];
	[accessory setValue:[NSNumber numberWithBool:NO] forKey:PageLayoutUpsideDownKey];
	[accessory setHeaderAndFooterEnabled:NO];
	[pageLayout addAccessoryController:accessory];
	[accessory release];
    [pageLayout beginSheetWithPrintInfo:tempPrintInfo modalForWindow:[self window] delegate:self didEndSelector:@selector(didEndPageLayout:returnCode:contextInfo:) contextInfo:(void *)tempPrintInfo];
}
- (void)didEndPageLayout:(NSPageLayout *)pageLayout returnCode:(int)result contextInfo:(void *)contextInfo {
    NSPrintInfo *tempPrintInfo = (NSPrintInfo *)contextInfo;

	if (result == NSModalResponseOK) {
		[printInfo release];
		printInfo = [tempPrintInfo copyWithZone:[self zone]];
		papersetSuccess = YES;
	}
	[tempPrintInfo release];
}

- (NSPrintInfo *)printInfo
{
	return printInfo;
}

- (NSString *)stringByConvertingString:(NSString *)localeString
{
	NSString *path = [[NSBundle mainBundle] pathForResource:@"Numeric" ofType:@"strings"];

	if (nil != path) {
		NSDictionary *numericDic = [NSDictionary dictionaryWithContentsOfFile:path];

		if (nil != numericDic) {
			NSString *aString;
			NSArray *anArray;
			NSMutableString *rStr = [NSMutableString string];
			NSRange range = NSMakeRange(0,1);

			while (range.location < [localeString length]) {
				aString = [localeString substringWithRange:range];
				anArray = [numericDic allKeysForObject:aString];
				if (0 < [anArray count])
					[rStr appendString:[anArray objectAtIndex:0]];
				else
					[rStr appendString:aString];
				range.location++;
			}
			return rStr;
		}
		else
			return [NSString stringWithString:localeString];
	}
	else
		return [NSString stringWithString:localeString];
}

- (void)indexDidChange:(NSNotification *)notification
{
	NSDictionary *aModel = [notification userInfo];
	NSString *empty = @"";
	NSString *country = (nil != [aModel objectForKey:CountryKey]) ? [aModel objectForKey:CountryKey] : empty;
	NSString *city = (nil != [aModel objectForKey:CityKey]) ? [aModel objectForKey:CityKey] : empty;
	NSString *town = (nil != [aModel objectForKey:TownKey]) ? [aModel objectForKey:TownKey] : empty;
	NSString *address = (nil != [aModel objectForKey:AddressKey]) ? [aModel objectForKey:AddressKey] : empty;
	NSString *zip = (nil != [aModel objectForKey:ZipKey]) ? [aModel objectForKey:ZipKey] : empty;
	NSString *name = (nil != [aModel objectForKey:NameKey]) ? [aModel objectForKey:NameKey] : empty;

	[self setAddress:[NSString stringWithFormat:@"%@%@%@",city,town,address] zip:zip country:country];
	[self setBarcode:[NSString stringWithFormat:@"%@%@%@",city,town,[self stringByConvertingString:address]] zip:zip country:country];
	[countryField setStringValue:country];
	[self setName:name];
}

@end
