//
//  MainDocument.m
//  ﾇPROJECTNAMEﾈ
//
//  Created by SatoAkira on ﾇDATEﾈ.
//  Copyright (c) 2003 SatoAkira. All rights reserved.
//

#import "MainDocument.h"
/* #import "HtmlFileAdaptor.h" */
#import "AddressBookAdaptor.h"
#import <ASPageLayoutAccessory/ASPageLayoutAccessory.h>
#import <libc.h> // for mktemp() //

// .enroll2 Keys //
NSString *IndexKey = @"Index";
NSString *RootIndexKey = @"RootIndex";
NSString *NameKey = @"Name";
NSString *NameKanaKey = @"NameKana";
NSString *PostKey = @"Post";
NSString *CountryKey = @"Country";
NSString *ZipKey = @"Zip";
NSString *PrefectureKey = @"Prefecture";
NSString *PrefectureKanaKey = @"PrefectureKana";
NSString *CityKey = @"City";
NSString *CityKanaKey = @"CityKana";
NSString *TownKey = @"Town";
NSString *TownKanaKey = @"TownKana";
NSString *AddressKey = @"Address";
NSString *TelNumberKey = @"TelNumber";
NSString *FaxNumberKey = @"FaxNumber";
NSString *PHSNumberKey = @"PHSNumber";
NSString *EmailKey = @"Email";
NSString *URLNameKey = @"URLName";
NSString *TypeKey = @"Type";
NSString *BirthDayKey = @"BirthDay";
NSString *OthersKey = @"Others";
NSString *LocateXKey = @"LocateX";
NSString *LocateYKey = @"LocateY";
NSString *LocateZKey = @"LocateZ";
NSString *ImagesFileNameKey = @"images";
NSString *SoundsFileNameKey = @"sounds";
NSString *EnrollImagesWrapperKey = @"imagesWrapper"; // NSFileWrapper *imagesWrapper //
NSString *EnrollSoundsWrapperKey = @"soundsWrapper"; // NSFileWrapper *soundsWrapper //
// .enroll2 Keys end.//

NSString *ModelKey = @"Model";
NSString *OldPictureFilenameKey = @"OldPictureFilenameKey";
NSString *OldSoundFilenameKey = @"OldSoundFilenameKey";

NSString *Enroll2DocumentType = @"com.SatoAkira.enroll2";
NSString *EnrollDocumentType = @"HTML Address list";
NSString *EnrollFilenameKey = @"enroll";
NSString *IndexDidChangeNotification = @"IndexDidChange";
NSString *ContextDidChangeNotification = @"ContextDidChange";
NSString *MailToDidChangeNotification = @"MailToDidChange";

@implementation MainDocument

- (instancetype)init {
    if (nil != (self = [super init])) {
		enrollDictionary = nil;
		namesDictionary = nil;
		imagesWrapper = nil;
		soundsWrapper = nil;
		NSPrintInfo *printInfo = [self printInfo];
		[printInfo setPaperSize:NSMakeSize(595.3,841.9)]; // これでpaperNameがiso-a4になる。setPaperName:をやると設定が壊される。ついでにプリンタ機種名まで設定される。 //
		NSRect imageable = [printInfo imageablePageBounds];
		NSSize dSize = [printInfo paperSize];
		[printInfo setOrientation:NSPaperOrientationPortrait]; // NSPortraitOrientation //
		[printInfo setLeftMargin:NSMinX(imageable)];
		[printInfo setRightMargin:dSize.width - NSMaxX(imageable)];
		[printInfo setTopMargin:NSMinY(imageable)];
		[printInfo setBottomMargin:dSize.height - NSMaxY(imageable)];
		[printInfo setVerticallyCentered:NO];
		[printInfo setHorizontallyCentered:YES];
		[printInfo setVerticalPagination:NSAutoPagination];
		[printInfo setHorizontalPagination:NSAutoPagination];
		[[printInfo dictionary] setObject:[NSNumber numberWithBool:YES] forKey:NSPrintHeaderAndFooter];
    }
    return self;
}

- (void)dealloc {
	[self removeWindowController:windowController];
	[windowController release];
	[enrollDictionary release];
	[namesDictionary release];
	[imagesWrapper release];
	[soundsWrapper release];
	[super dealloc];
}

- (void)makeWindowControllers {
	windowController = [[MainWindowController alloc] init];
	[windowController setShouldCloseDocument:YES];
	[self addWindowController:windowController];
}

// ---------override--------- //
+ (BOOL)autosavesInPlace {
	return YES;
}
- (BOOL)canAsynchronouslyWriteToURL:(NSURL *)url ofType:(NSString *)typeName forSaveOperation:(NSSaveOperationType)saveOperation {
	// Returns whether the receiver can concurrently write to a file or file package located by a URL, formatted for a specific type, for a specific kind of save operation. NO by default; subclasses can override to return YES, thereby enabling asynchronous writing. //
	return YES;
}
- (void)documentController:(NSDocumentController *)docController didCloseAll:(BOOL)didCloseAll contextInfo:(void *)contextInfo {
	// AppControllerからNSDocumentController の closeAllDocumentsWithDelegate:を使って[NSApp terminate]からやらせるときには有効にする。しかし、NSDocumentControllerはこいつを1回しか実行しないので使えない。複数のドキュメントが開かれていたときは1回しか呼ばれない。ここからMainDocumentのdeallocは呼ばれない。 //
	if (YES == didCloseAll) {
		if (nil != windowController) {
			[self removeWindowController:windowController];
			[windowController release];
		}
		[enrollDictionary release];
		[namesDictionary release];
		[imagesWrapper release];
		[soundsWrapper release];
	}
}
- (NSDictionary *)fileWrapperDictionary {
	// SoundFile,PictureFileはOldSoundFilenameKey,OldPictureFilenameKeyの名前とIndexKeyとの整合性をとるようにここでリネームする。他のところでは絶対にリネームをしないこと。OldSoundFilenameKey,OldPictureFilenameKeyの値はディスクには書かない。また絶対パスであったならばここでコピーをしてSoundFilenameKey,PictureFilenameKeyの値を書き換える。これも他のところでは絶対に値を書き換えないこと。削除されたindexがあればfileWrapperに含ませないようにすれば、自動的に消え去る。 //
	NSString *filename;
	NSFileWrapper *fileWrapper;
	NSMutableDictionary *fileWrappersDic = [NSMutableDictionary dictionary];
	NSError *err;

	if (nil != imagesWrapper)
		[fileWrappersDic setObject:imagesWrapper forKey:ImagesFileNameKey];
	if (nil != soundsWrapper)
		[fileWrappersDic setObject:soundsWrapper forKey:SoundsFileNameKey];
	err = nil;
	if (nil != (fileWrapper = [[NSFileWrapper alloc] initRegularFileWithContents:[NSPropertyListSerialization dataWithPropertyList:enrollDictionary format:NSPropertyListXMLFormat_v1_0 options:NSPropertyListImmutable error:&err]])) {
		filename = [NSString stringWithString:EnrollFilenameKey];
		[fileWrappersDic setObject:fileWrapper forKey:filename];
		[fileWrapper release];
		return fileWrappersDic;
	}
	else
		return nil;
}
// ---------override--------- //
- (NSFileWrapper *)fileWrapperOfType:(NSString *)typeName error:(NSError **)outError {
	[self unblockUserInteraction]; // Available in Mac OS X v10.7 and later. Unblocks the main thread during asynchronous saving. //
	NSString *myErrorDomain = [[[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleIdentifier"] stringByAppendingPathExtension:@"ErrorDomain"];
	NSWorkspace *workspace = [NSWorkspace sharedWorkspace];

	if (nil != enrollDictionary) {
		if (YES == [workspace type:typeName conformsToType:Enroll2DocumentType]) {
			NSDictionary *fwDic;
			if (nil != (fwDic = [self fileWrapperDictionary])) {
				NSFileWrapper *wrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:fwDic];
				if (nil != wrapper)
					return [wrapper autorelease];
				else {
					if ((NSError **)NULL != outError)
						*outError = [NSError errorWithDomain:myErrorDomain code:NSFileReadCorruptFileError userInfo:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"The rapper to preserve it was not normally generable.",@"保存するためのラッパーが正常に生成できませんでした。"),NSLocalizedRecoverySuggestionErrorKey,nil]];
					return nil;
				}
			}
			else {
				if ((NSError **)NULL != outError)
					*outError = [NSError errorWithDomain:myErrorDomain code:NSFileReadCorruptFileError userInfo:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"The rapper to preserve it was not normally generable.",@"保存するためのラッパーが正常に生成できませんでした。"),NSLocalizedRecoverySuggestionErrorKey,nil]];
				return nil;
			}
		}
		else {
			if ((NSError **)NULL != outError)
				*outError = [NSError errorWithDomain:myErrorDomain code:NSFileReadCorruptFileError userInfo:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"type is bad.",@""),NSLocalizedRecoverySuggestionErrorKey,nil]];
			return nil;
		}
	}
	else {
        NSAlert *alert = [[NSAlert alloc] init];
        // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
        [alert setMessageText:NSLocalizedString(@"Enroll is empty.",@"名簿データが存在しません。")];
        [alert setInformativeText:(nil != [self fileURL]) ? [[self fileURL] path] : NSLocalizedString(@"Untitled",@"名称未設定")];
        [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
        [alert beginSheetModalForWindow:[windowController window] completionHandler:^(NSInteger result) {
            // 何もやることがない //
        }];
        [alert release];
		/* NSBeginInformationalAlertSheet(NSLocalizedString(@"Enroll is empty.",@"名簿データが存在しません。"),nil,nil,nil,[windowController window],self,NULL,NULL,NULL,(nil != [self fileURL]) ? [[[self fileURL] absoluteString] stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding] : NSLocalizedString(@"Untitled",@"名称未設定"),nil); */
		if ((NSError **)NULL != outError)
			*outError = [NSError errorWithDomain:myErrorDomain code:NSFileReadCorruptFileError userInfo:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"type is bad.",@""),NSLocalizedRecoverySuggestionErrorKey,nil]];
		return nil;
	}
}

// ---------override--------- printDocument:がfirstResponderに接続され、そのfirstResponderにprintDocument:が実装されていないとこれが呼ばれる。NSPrintOperationに対してrunOperationModalForWindow:をしなくても[thePrintOperation setShowsPrintPanel:showPrintPanel];[self runModalPrintOperation:thePrintOperation delegate:delegate didRunSelector:didPrintSelector contextInfo:contextInfo];される。printSettingsのcountは0のことが多いし何が入っているのかわからない。 //
- (NSPrintOperation *)printOperationWithSettings:(NSDictionary *)printSettings error:(NSError **)outError {
	NSPrintOperation *pOpe = [NSPrintOperation printOperationWithView:[[windowController window] contentView] printInfo:[self printInfo]];

	if (nil == pOpe) {
		NSString *myErrorDomain = [[[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleIdentifier"] stringByAppendingPathExtension:@"ErrorDomain"];

		if ((NSError **)NULL != outError)
			*outError = [NSError errorWithDomain:myErrorDomain code:NSFileWriteUnknownError userInfo:[NSDictionary dictionaryWithObjectsAndKeys:@"NSPrintOperation can't create print operation.",NSLocalizedRecoverySuggestionErrorKey,nil]];
		return nil;
	}
	else {
		[pOpe setCanSpawnSeparateThread:YES];
		[[[pOpe printInfo] dictionary] addEntriesFromDictionary:printSettings];
		return pOpe;
	}
}

/* - (BOOL)fileManager:(NSFileManager *)manager shouldProceedAfterError:(NSDictionary *)errorInfo {
	return (NSAlertDefaultReturn == NSRunAlertPanel([NSString stringWithFormat:NSLocalizedString(@"File operation error:%@",@""),[errorInfo objectForKey:@"Error"]],[NSString stringWithFormat:NSLocalizedString(@"with file:%@",@""),[errorInfo objectForKey:@"Path"]],NSLocalizedString(@"Proceed",@""),NSLocalizedString(@"Stop",@""),nil)) ? YES : NO; // YES:継続  NO:中断 //
} */

- (void)makeNamesDictionary {
	NSString *index,*name;
	NSArray *indexes;
	NSMutableDictionary *tempDic = [NSMutableDictionary dictionary];
	NSEnumerator *enrollEnumerator = [[enrollDictionary allKeys] objectEnumerator];

	while (nil != (index = [enrollEnumerator nextObject])) {
		[tempDic setObject:[[enrollDictionary objectForKey:index] objectForKey:NameKey] forKey:index];
	}
	indexes = [tempDic keysSortedByValueUsingSelector:@selector(compare:)];
	[namesDictionary release];
	namesDictionary = [[NSMutableDictionary alloc] initWithCapacity:10];
    NSUInteger i = 0;
	for (i = 0; i < [indexes count]; i++) {
		index = [indexes objectAtIndex:i];
		name = [tempDic objectForKey:index];
		[namesDictionary setObject:[NSDictionary dictionaryWithObjectsAndKeys:name,NameKey,index,IndexKey,nil] forKey:[NSString stringWithFormat:@"%lu",i]]; // 同姓同名があるので名前をキーにできない //
	}
}

// ---------override--------- //
+ (BOOL)canConcurrentlyReadDocumentsOfType:(NSString *)typeName {
	// Your NSDocument subclass can implement this method to return YES to enable loading of documents concurrently, using background threads. When this facility is enabled in this way, initWithContentsOfURL:ofType:error: executes on a background thread when opening files via the Open panel or from the Finder. This allows concurrent reading of multiple documents and also allows the application to be responsive while reading a large document. //
	// The default implementation of this method returns NO. A subclass override should return YES only for document types whose reading is thread-safe, as described in “Multicore Considerations.” You should disable undo registration during document reading, which is a good idea even in the absence of concurrency. //
	// If you are checking the current Apple Event for a search string, you should not enable concurrent document opening, because code handling a document opening triggered by an Apple Event cannot get the current Apple Event. This happens because the event is suspended until all documents are read to enable correct reporting of success or error. //
	return YES;
}
- (BOOL)readFromFileWrapper:(NSFileWrapper *)fileWrapper ofType:(NSString *)typeName error:(NSError **)outError {
	// revertDocumentToSaved:からもここが呼ばれる。 //
	NSString *myErrorDomain = [[[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleIdentifier"] stringByAppendingPathExtension:@"ErrorDomain"];
	NSWorkspace *workspace = [NSWorkspace sharedWorkspace];

	[enrollDictionary release];
	enrollDictionary = nil;
	if ((YES == [workspace type:typeName conformsToType:Enroll2DocumentType]) && (YES == [fileWrapper isDirectory])) {
		NSFileWrapper *oneFileWrapper;
		NSDictionary *fileWrappersDic = [fileWrapper fileWrappers];
		[enrollDictionary release];
		enrollDictionary = nil;
		[imagesWrapper release];
		imagesWrapper = nil;
		[soundsWrapper release];
		soundsWrapper = nil;
		for (NSString *filename in fileWrappersDic) {
			oneFileWrapper = [fileWrappersDic objectForKey:filename];
			if ((YES == [filename isEqualToString:EnrollFilenameKey]) && (YES == [oneFileWrapper isRegularFile])) {
				// enrollDictionary = [[NSMutableDictionary alloc] initWithContentsOfURL:[NSURL URLWithString:filename relativeToURL:[self fileURL]]]; とするとNSDocumentのバージョン管理で前のバージョンを読むときにエラーが発生してenrollDictionaryがnilになる。fileURLを使うことがおかしいのでエラーが発生する。古いバージョンを読むときのfileURLは通常の物と違う。 //
				NSPropertyListFormat form; // 書き込むときはNSPropertyListXMLFormat_v1_0を使用 //
				NSError *err = nil;
				id plist = [NSPropertyListSerialization propertyListWithData:[oneFileWrapper regularFileContents] options:NSPropertyListImmutable format:&form error:&err];
				if ((nil != plist) && (YES == [plist isKindOfClass:[NSDictionary class]]))
					enrollDictionary = [[NSMutableDictionary alloc] initWithDictionary:(NSDictionary *)plist];
				if (nil != enrollDictionary)
					[self makeNamesDictionary];
				[[NSNotificationCenter defaultCenter] postNotificationName:ContextDidChangeNotification object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys:[NSString string],IndexKey,[NSDictionary dictionary],ModelKey,nil]];
			}
			else if ((YES == [filename isEqualToString:ImagesFileNameKey]) && (YES == [oneFileWrapper isDirectory]))
				imagesWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:[oneFileWrapper fileWrappers]]; // 新形式 //
			else if ((YES == [filename isEqualToString:SoundsFileNameKey]) && (YES == [oneFileWrapper isDirectory]))
				soundsWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:[oneFileWrapper fileWrappers]]; // 新形式 //
		}
		if (nil != enrollDictionary) {
			// 旧形式のデータと互換性をとるため //
			NSDictionary *aModel;
			NSString *imageFilename,*soundFilename;
			NSFileWrapper *wrapper;
			NSMutableDictionary *imagesWrapperDic = [NSMutableDictionary dictionary];
			NSMutableDictionary *soundsWrapperDic = [NSMutableDictionary dictionary];
			if ((nil == imagesWrapper) && (nil == soundsWrapper)) {
				for (NSString *index in enrollDictionary) {
					aModel = [enrollDictionary objectForKey:index];
					if ((nil != (imageFilename = [aModel objectForKey:@"PictureFilename"])) && (NO == [imageFilename isEqualToString:@""]) && (nil != (oneFileWrapper = [fileWrappersDic objectForKey:imageFilename])) && (YES == [oneFileWrapper isRegularFile])) {
						wrapper = [[NSFileWrapper alloc] initRegularFileWithContents:[oneFileWrapper regularFileContents]];
						[imagesWrapperDic setObject:wrapper forKey:imageFilename];
						[wrapper release];
					}
					if ((nil != (soundFilename = [aModel objectForKey:@"SoundFilename"])) && (NO == [soundFilename isEqualToString:@""]) && (nil != (oneFileWrapper = [fileWrappersDic objectForKey:soundFilename])) && (YES == [oneFileWrapper isRegularFile])) {
						wrapper = [[NSFileWrapper alloc] initRegularFileWithContents:[oneFileWrapper regularFileContents]];
						[soundsWrapperDic setObject:wrapper forKey:soundFilename];
						[wrapper release];
					}
				}
				if (0 < [imagesWrapperDic count]) {
					imagesWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:imagesWrapperDic];
					[imagesWrapper setPreferredFilename:ImagesFileNameKey];
				}
				if (0 < [soundsWrapperDic count]) {
					soundsWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:soundsWrapperDic];
					[soundsWrapper setPreferredFilename:SoundsFileNameKey];
				}
			}
			// 旧形式のデータと互換性をとるための処理終了 //
			return YES;
		}
		else {
			if ((NSError **)NULL != outError)
				*outError = [NSError errorWithDomain:myErrorDomain code:NSFileReadUnknownError userInfo:[NSDictionary dictionaryWithObject:NSLocalizedString(@"It failed in reading. Work cannot begin.",@"") forKey:NSLocalizedRecoverySuggestionErrorKey]];
				return NO;
			}
	}
	else {
		if ((NSError **)NULL != outError)
			*outError = [NSError errorWithDomain:myErrorDomain code:NSFileReadUnknownError userInfo:[NSDictionary dictionaryWithObject:NSLocalizedString(@"type error. It failed in reading. Work cannot begin.",@"") forKey:NSLocalizedRecoverySuggestionErrorKey]];
		return NO;
	}
}

- (MainWindowController *)windowController {
	return windowController;
}
- (NSDictionary *)enrollDictionary {
	return enrollDictionary;
}
- (NSArray *)indexes {
	return (nil != enrollDictionary) ? [[enrollDictionary allKeys]  sortedArrayUsingSelector:@selector(compare:)] : [NSArray array];
}

- (NSString *)nameAtNumber:(NSUInteger)number {
	return [[namesDictionary objectForKey:[NSString stringWithFormat:@"%lu",number]] objectForKey:NameKey];
}
- (NSArray *)names {
	// 検索用に使われる //
	if (nil != namesDictionary) {
		NSUInteger i;
		NSArray *keys = [namesDictionary allKeys];
		NSMutableArray *nameArray = [NSMutableArray array];

        for (i = 0; i < [keys count]; i++)
			/* [nameArray addObject:[[namesDictionary objectForKey:[keys objectAtIndex:i]] objectForKey:NameKey]]; */
			[nameArray addObject:[[namesDictionary objectForKey:[NSString stringWithFormat:@"%lu",i]] objectForKey:NameKey]];
		return nameArray;
	}
	else
		return nil;
}
- (NSUInteger)numberOfName:(NSString *)aString {
	// 同姓同名があった場合にはおかしくなる //
	NSUInteger i;
	NSArray *keys = [namesDictionary allKeys];

	for (i = 0; i < [keys count]; i++) {
		if (YES == [aString isEqualToString:[[namesDictionary objectForKey:[NSString stringWithFormat:@"%lu",i]] objectForKey:NameKey]])
			break;
	}
	if (i < [keys count])
		return i;
	else
		return NSNotFound;
}
- (NSArray *)modelOfName:(NSString *)aString {
	NSArray *names = [self names];

	if (nil != names) {
		NSUInteger number = [names indexOfObject:aString];

		if (NSNotFound != number) {
			/* NSString *index = [[namesDictionary objectForKey:[NSNumber numberWithUnsignedInteger:number]] objectForKey:IndexKey]; */
			NSString *index = [[namesDictionary objectForKey:[NSString stringWithFormat:@"%lu",number]] objectForKey:IndexKey];

			return [NSArray arrayWithObjects:index,[enrollDictionary objectForKey:index],nil];
		}
		else
			return [NSArray arrayWithObjects:@"",[NSDictionary dictionary],nil];
	}
	else
		return nil;
}
- (NSDictionary *)namesDictionary {
	return namesDictionary;
}

- (IBAction)runPageLayout:(id)sender {
	NSPrintInfo *tempPrintInfo = [[self printInfo] copy];
	NSPageLayout *pageLayout = [NSPageLayout pageLayout];
#ifdef WIN32
	float lm = [tempPrintInfo leftMargin] * 2.540000;
	float rm = [tempPrintInfo rightMargin] * 2.540000;
	float tm = [tempPrintInfo topMargin] * 2.540000;
	float bm = [tempPrintInfo bottomMargin] * 2.540000;

	[tempPrintInfo setLeftMargin:lm];
	[tempPrintInfo setRightMargin:rm];
	[tempPrintInfo setTopMargin:tm];
	[tempPrintInfo setBottomMargin:bm];
#endif
	ASPageLayoutAccessory *accessory = [[ASPageLayoutAccessory alloc] initWithNibName:PageLayoutNibName bundle:nil];
	[accessory setValue:[NSNumber numberWithBool:YES] forKey:PageLayoutUpsideDownKey];
	[pageLayout addAccessoryController:accessory];
	[accessory release];
	[pageLayout beginSheetWithPrintInfo:tempPrintInfo modalForWindow:[windowController window] delegate:self didEndSelector:@selector(pageLayoutDidEnd:returnCode:contextInfo:) contextInfo:NULL];
	[tempPrintInfo release];
}
- (void)pageLayoutDidEnd:(NSPageLayout *)pageLayout returnCode:(int)returnCode contextInfo:(void *)contextInfo {
	if (NSModalResponseOK == returnCode) {
#ifdef WIN32
		NSPrintInfo *tempPrintInfo = [pageLayout printInfo];
		float lm = [tempPrintInfo leftMargin] / 2.540000;
		float rm = [tempPrintInfo rightMargin] / 2.540000;
		float tm = [tempPrintInfo topMargin] / 2.540000;
		float bm = [tempPrintInfo bottomMargin] / 2.540000;

		[tempPrintInfo setLeftMargin:lm];
		[tempPrintInfo setRightMargin:rm];
		[tempPrintInfo setTopMargin:tm];
		[tempPrintInfo setBottomMargin:bm];
		[self setPrintInfo:tempPrintInfo];
#else
		[self setPrintInfo:[pageLayout printInfo]];
		[self updateChangeCount:NSChangeUndone];
#endif
	}
}

- (NSDictionary *)reverseLocalizedDictionary {
	return [NSDictionary dictionaryWithObjectsAndKeys:IndexKey,NSLocalizedString(IndexKey,@""),RootIndexKey,NSLocalizedString(RootIndexKey,@""),NameKey,NSLocalizedString(NameKey,@""),NameKanaKey,NSLocalizedString(NameKanaKey,@""),PostKey,NSLocalizedString(PostKey,@""),CountryKey,NSLocalizedString(CountryKey,@""),ZipKey,NSLocalizedString(ZipKey,@""),PrefectureKey,NSLocalizedString(PrefectureKey,@""),PrefectureKanaKey,NSLocalizedString(PrefectureKanaKey,@""),CityKey,NSLocalizedString(CityKey,@""),CityKanaKey,NSLocalizedString(CityKanaKey,@""),TownKey,NSLocalizedString(TownKey,@""),TownKanaKey,NSLocalizedString(TownKanaKey,@""),AddressKey,NSLocalizedString(AddressKey,@""),TelNumberKey,NSLocalizedString(TelNumberKey,@""),FaxNumberKey,NSLocalizedString(FaxNumberKey,@""),PHSNumberKey,NSLocalizedString(PHSNumberKey,@""),EmailKey,NSLocalizedString(EmailKey,@""),URLNameKey,NSLocalizedString(URLNameKey,@""),TypeKey,NSLocalizedString(TypeKey,@""),BirthDayKey,NSLocalizedString(BirthDayKey,@""),OthersKey,NSLocalizedString(OthersKey,@""),LocateXKey,NSLocalizedString(LocateXKey,@""),LocateYKey,NSLocalizedString(LocateYKey,@""),LocateZKey,NSLocalizedString(LocateZKey,@""),nil];
}
- (NSString *)reverseLocalizedString:(NSString *)aString {
	NSString *rStr = [[self reverseLocalizedDictionary] objectForKey:aString];

	return (rStr != nil) ? rStr : aString;
}

- (BOOL)changeIndex:(NSString *)oldIndex to:(NSString *)newIndex {
	// この段階ではPictureFilenameKey,SoundFilenameKeyの値は元のままである。ファイルの名前もいじっていない。絶対パスのときもある。 //
	NSFileWrapper *wrapper;
	NSMutableDictionary *wrappers = [NSMutableDictionary dictionaryWithDictionary:[imagesWrapper fileWrappers]];
	for (NSString *imageFilename in wrappers) {
		if (YES == [oldIndex isEqualToString:[imageFilename stringByDeletingPathExtension]]) {
			wrapper = [[NSFileWrapper alloc] initRegularFileWithContents:[[wrappers objectForKey:imageFilename] regularFileContents]];
			[wrappers setObject:wrapper forKey:[newIndex stringByAppendingPathExtension:[imageFilename pathExtension]]];
			[wrappers removeObjectForKey:imageFilename];
			[wrapper release];
			[imagesWrapper release];
			imagesWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:wrappers];
			break;
		}
	}
	wrappers = [NSMutableDictionary dictionaryWithDictionary:[soundsWrapper fileWrappers]];
	for (NSString *soundFilename in wrappers) {
		if (YES == [oldIndex isEqualToString:[soundFilename stringByDeletingPathExtension]]) {
			wrapper = [[NSFileWrapper alloc] initRegularFileWithContents:[[wrappers objectForKey:soundFilename] regularFileContents]];
			[wrappers setObject:wrapper forKey:[newIndex stringByAppendingPathExtension:[soundFilename pathExtension]]];
			[wrappers removeObjectForKey:soundFilename];
			[wrapper release];
			[soundsWrapper release];
			soundsWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:wrappers];
			break;
		}
	}

	NSMutableDictionary *aModel,*newModel;
	NSMutableDictionary *tempDic = [NSMutableDictionary dictionaryWithDictionary:enrollDictionary];
	for (NSString *key in tempDic) {
		aModel = [NSMutableDictionary dictionaryWithDictionary:[tempDic objectForKey:key]];
		if (YES == [oldIndex isEqualToString:[aModel objectForKey:RootIndexKey]]) {
			[aModel setObject:newIndex forKey:RootIndexKey];
			[tempDic setObject:aModel forKey:key];
		}
	}
	newModel = [NSMutableDictionary dictionaryWithDictionary:[tempDic objectForKey:oldIndex]];
	[tempDic setObject:newModel forKey:newIndex];
	[tempDic removeObjectForKey:oldIndex];
	[enrollDictionary release];
	enrollDictionary = [[NSDictionary alloc] initWithDictionary:tempDic];
	[self makeNamesDictionary]; // IndexKeyだけを入れ替える方法もあるが面倒くさいので、0から作り直す。大量のデータになると速度が落ちるが、滅多に呼ばれない操作なのでこのままにする。 //
	[self updateChangeCount:NSChangeDone];
	[[NSNotificationCenter defaultCenter] postNotificationName:ContextDidChangeNotification object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys:newIndex,IndexKey,newModel,ModelKey,nil]];
	return YES;
}

- (BOOL)addModel:(NSDictionary *)aValue picture:(NSURL *)imageURL pictureData:(NSData *)pData sound:(NSURL *)soundURL soundData:(NSData *)sData forKey:(NSString *)index {
	NSString *actionName;
	if (nil == enrollDictionary) {
		enrollDictionary = [[NSDictionary alloc] init];
		actionName = NSLocalizedString(@"Add",@"");
	}
	else
		actionName = (nil == [enrollDictionary objectForKey:index]) ? NSLocalizedString(@"Add",@"") : NSLocalizedString(@"Change",@"");
	if (nil != enrollDictionary) {
		BOOL hasNameChanged;
		NSMutableDictionary *tempDic = [NSMutableDictionary dictionaryWithDictionary:enrollDictionary];

		if (YES == [actionName isEqualToString:NSLocalizedString(@"Change",@"")])
			hasNameChanged = (YES == [[aValue objectForKey:NameKey] isEqualToString:[[enrollDictionary objectForKey:index] objectForKey:NameKey]]) ? NO : YES;
		else
			hasNameChanged = YES;
		[tempDic setObject:aValue forKey:index];
		[enrollDictionary release];
		enrollDictionary = [[NSDictionary alloc] initWithDictionary:tempDic];
		NSMutableDictionary *wrappers = [NSMutableDictionary dictionaryWithDictionary:[imagesWrapper fileWrappers]];
		if ((nil != imageURL) && (nil != pData)) {
			for (NSString *imageFilename in wrappers) {
				if (YES == [index isEqualToString:[imageFilename stringByDeletingPathExtension]]) {
					[wrappers removeObjectForKey:imageFilename];
					break;
				}
			}
			NSFileWrapper *wrapper = [[NSFileWrapper alloc] initRegularFileWithContents:pData];
			[wrappers setObject:wrapper forKey:[index stringByAppendingPathExtension:[imageURL pathExtension]]];
			[wrapper release];
		}
		else {
			if ((nil == imageURL) && (nil == pData)) {
				for (NSString *imageFilename in wrappers) {
					if (YES == [index isEqualToString:[imageFilename stringByDeletingPathExtension]]) {
						[wrappers removeObjectForKey:imageFilename];
						break;
					}
				}
			}
		}
		[imagesWrapper release];
		imagesWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:wrappers];
		wrappers = [NSMutableDictionary dictionaryWithDictionary:[soundsWrapper fileWrappers]];
		if ((nil != soundURL) && (nil != sData)) {
			for (NSString *soundFilename in wrappers) {
				if (YES == [index isEqualToString:[soundFilename stringByDeletingPathExtension]]) {
					[wrappers removeObjectForKey:soundFilename];
					break;
				}
			}
			NSFileWrapper *wrapper = [[NSFileWrapper alloc] initRegularFileWithContents:sData];
			[wrappers setObject:wrapper forKey:[index stringByAppendingPathExtension:[soundURL pathExtension]]];
			[wrapper release];
		}
		else {
			if ((nil == soundURL) && (nil == sData)) {
				for (NSString *soundFilename in wrappers) {
					if (YES == [index isEqualToString:[soundFilename stringByDeletingPathExtension]]) {
						[wrappers removeObjectForKey:soundFilename];
						break;
					}
				}
			}
		}
		[soundsWrapper release];
		soundsWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:wrappers];
		if (YES == hasNameChanged)
			[self makeNamesDictionary];
		[self updateChangeCount:NSChangeDone];
		[[NSNotificationCenter defaultCenter] postNotificationName:ContextDidChangeNotification object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys:index,IndexKey,aValue,ModelKey,nil]];
		return YES;
	}
	else
		return NO;
}
- (BOOL)deleteIndex:(NSString *)oldIndex {
	if (nil != [enrollDictionary objectForKey:oldIndex]) {
		NSMutableDictionary *tempDic = [NSMutableDictionary dictionaryWithDictionary:enrollDictionary];
		[tempDic removeObjectForKey:oldIndex];
		[enrollDictionary release];
		enrollDictionary = [[NSDictionary alloc] initWithDictionary:tempDic];
		[self makeNamesDictionary]; // 同姓同名があったりするので、0から作り直す。 //
		NSMutableDictionary *wrappers = [NSMutableDictionary dictionaryWithDictionary:[imagesWrapper fileWrappers]];
		for (NSString *imageFilename in wrappers) {
			if (YES == [oldIndex isEqualToString:[imageFilename stringByDeletingPathExtension]]) {
				[wrappers removeObjectForKey:imageFilename];
				[imagesWrapper release];
				imagesWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:wrappers];
				break;
			}
		}
		wrappers = [NSMutableDictionary dictionaryWithDictionary:[soundsWrapper fileWrappers]];
		for (NSString *soundFilename in wrappers) {
			if (YES == [oldIndex isEqualToString:[soundFilename stringByDeletingPathExtension]]) {
				[wrappers removeObjectForKey:soundFilename];
				[soundsWrapper release];
				soundsWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers:wrappers];
				break;
			}
		}
		[[NSNotificationCenter defaultCenter] postNotificationName:ContextDidChangeNotification object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys:oldIndex,IndexKey,[NSDictionary dictionary],ModelKey,nil]];
		return YES;
	}
	else
		return NO;
}

- (NSDictionary *)selfDictionary {
	NSRange range;
	NSDictionary *aDic;
	NSDictionary *rDic = nil;

	for (NSString *key in enrollDictionary) {
		aDic = [enrollDictionary objectForKey:key];
		range = [[aDic objectForKey:OthersKey] rangeOfString:@"\"self\""];
		if (0 < range.length) {
			rDic = aDic;
			break;
		}
	}
	return rDic;
}

@end
