//
//  MainWindowController.m
//  Enroll
//
//  Created by 佐藤 昭 on Tue Jul 16 2002.
//  Copyright (c) 2002 SatoAkira. All rights reserved.
//

#import "MainWindowController.h"
#import "EnrollFunctions.h"
#import "MainDocument.h"
#import "HagakiPrint.h"
#import "Report.h"
#import "PostalFind.h"
/* #import "MailSend.h" */
#import "AddressBookAdaptor.h"

#define UNradian 1
#define UNdegree 2
#define UNdms 3
#define UNsecond 4
#define UNhms 5

#define PAI   3.14159265358979323846
#define PAI2  6.28318530717958647692
#define LOW   206264.8062470964 /* 四捨五入はされないので最後は切り上げておく */

@interface MainWindowController () <NSComboBoxDataSource,NSWindowDelegate>

@property IBOutlet NSPopover *postalPopover;
@property IBOutlet NSPopover *changeIndexPopover;

@end

@implementation MainWindowController

double ASRound(int p,double x)
/* double型のxを精度pで四捨五入又は切り捨てしてその結果を返す。*/
/* pが負数のとき切り捨て。pが-100以下のときは整数で切り捨て。*/
{
	NSInteger i,pt;
	double xt,sgn;
	sgn = (x < 0) ? -1 : 1;
	if( (pt = abs(p)) > 99 ) pt = 0;
	xt = fabs(x);
	for( i = 0; i < pt; i++ ) xt *= 10;
	if( p >= 0 ) xt = floor(xt + 0.5);
	else {
		/* floor(),modf()などの関数では誤差が生じる。例えば、x = 8.08 * 8.25 = 66.6000など */
		/* 0.9999...の精度はpによって変化させるのが妥当だと思うがやりかたが分からない。 */
		if( modf(xt,&xt) > 0.999999999999 ) xt += 1.0;
	}
	for( i = 0; i < pt; i++ ) xt /= 10;
	return( sgn * xt );
}

double rtod(double x)
/* ラジアンxをDDD.MMSSSSSS表現に変換 */
{
	double a,b;
	NSInteger sgn;
	if( x < 0.0 ) { x = fabs(x); sgn = -1; }
	else sgn = 1;
	x = modf( modf(x * 57.29577951308232997, &a) * 60.0, &b ) * 0.006;
	x += a + b * 0.01;
	return( (sgn == -1) ? -x : x );
}

NSString *angleForm(int radianOrDegree,double value,NSString *form) {
	double x,a,b,ss;
	NSInteger m,d,sgn;
	NSString *formats;
	NSRange fRange;
	static double error = 1.0e-11;	/* modf()が1.0を返してしまうことを防ぐ為の数値 */
	
	switch( radianOrDegree ) {
	case UNradian:
		x = rtod(value);
		break;
	case UNdegree:
		x = rtod(value / PAI2);
		break;
	case UNsecond:
		x = rtod(value / LOW);
		break;
	default:
		x = value;
		break;
	}
	if( (nil == form) || (YES == [form isEqualToString:@""]) ) {
		if( UNhms == radianOrDegree )
			formats = @"%2d:%02d:%2.0f";
		else
			formats = [NSString stringWithFormat:@"%@%@%@",@"%4d",NSLocalizedString(@"-",@"°"),@"%02d\'%02.0f\""];
	}
	else
		formats = [NSString stringWithString:form];
	if( x < 0.0 ) {
		x = fabs(x);
		sgn = -1;
	}
	else
		sgn = 1;
	fRange = [formats rangeOfString:@"f" options:NSBackwardsSearch];
	if( 0 < fRange.length ) {
		NSRange periodRange = [formats rangeOfString:@"." options:NSBackwardsSearch range:NSMakeRange(0,fRange.location)];

		if( (0 < periodRange.length) && (fRange.location > periodRange.location) )
			ss = ASRound([[formats substringWithRange:NSMakeRange(periodRange.location + 1,fRange.location - periodRange.location - 1)] intValue],modf( modf(x,&a) * 100.0 + error, &b ) * 100.0);
		else
			ss = modf( modf(x,&a) * 100.0 + error, &b ) * 100.0;
	}
	else
		ss = modf( modf(x,&a) * 100.0 + error, &b ) * 100.0;
	m = (int)b;
	d = (int)a;
	if( ss >= 60.0 ) {
		ss -= 60.0;
		m++;
	}
	if( m >= 60 ) {
		m -= 60;
		d++;
	}
	if( (0 == d) && (-1 == sgn) ) {
		NSMutableString *returnString = [NSMutableString stringWithFormat:formats,-1,m,ss];
		NSRange range = [returnString rangeOfString:@"1"];
		[returnString replaceCharactersInRange:range withString:@"0"];
		return returnString;
	}
	else
		return [NSString stringWithFormat:formats,sgn * d,m,ss];
}

- (instancetype)init {
	if (nil != (self = [super initWithWindowNibName:@"MainDocument"])) {
		soundController = nil;
	}
	return self;
}
- (void)dealloc {
	[soundController release];
	[super dealloc];
}
- (void)windowDidLoad {
	NSAttributedString *attributedTitle;
	NSMutableParagraphStyle *parStyle = [[NSMutableParagraphStyle alloc] init];

	[[indexCombo cell] setSendsActionOnEndEditing:NO];
	[[indexCombo cell] setFont:[NSFont fontWithName:NSLocalizedString(@"ScreenFont",@"") size:13.0]];
	[indexCombo setUsesDataSource:YES];
	[indexCombo setDataSource:self];
	[indexCombo setCompletes:YES];
	[indexCombo setNumberOfVisibleItems:(20 > [[[self document] indexes] count]) ? [[[self document] indexes] count] : 20];
	[indexTitleField setStringValue:NSLocalizedString(@"Index",@"コード")];
	[indexChangeButton setTitle:NSLocalizedString(@"ChangeIndex",@"コード変更")];
	[[rootIndexForm titleAtIndex:0] setStringValue:NSLocalizedString(@"RootIndex",@"関連コード")];
	[[rootIndexForm cellAtIndex:0] setFont:[NSFont fontWithName:NSLocalizedString(@"ScreenFont",@"") size:13.0]];
	[[[rootIndexForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[rootIndexForm calcSize];
	[[nameCombo cell] setSendsActionOnEndEditing:YES];
	[[nameCombo cell] setFont:[NSFont fontWithName:NSLocalizedString(@"ScreenFont",@"") size:13.0]];
	[nameCombo setUsesDataSource:YES];
	[nameCombo setDataSource:self];
	[nameCombo setCompletes:YES];
	[nameCombo setNumberOfVisibleItems:(20 > [[[self document] indexes] count]) ? [[[self document] indexes] count] : 20];
	[nameTitleField setStringValue:NSLocalizedString(@"Name",@"氏名")];
	[nameKanaField setStringValue:NSLocalizedString(@"NameKana",@"")];
	[[postForm titleAtIndex:0] setStringValue:NSLocalizedString(@"Post",@"所属部署")];
	[[[postForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[postForm calcSize];
	[countryCombo removeAllItems];
	[countryCombo setUsesDataSource:NO];
	[countryCombo addItemsWithObjectValues:[countryCodeDic() allKeys]];
	[countryCombo setObjectValue:NSLocalizedString(@"LocalCountry",@"")];
	[countryCombo setToolTip:NSLocalizedString(@"Country",@"")];
	[countryCombo setCompletes:YES];
	[[zipForm titleAtIndex:0] setStringValue:NSLocalizedString(@"Zip",@"郵便番号")];
	[[[zipForm cellAtIndex:0] cell] setSendsActionOnEndEditing:NO]; // NOにしないと非常に訳の分からない動きになる。NSFormCellからのActionにしなければならない。NSFormからのActionにするとエラーが発生する。 //
	[zipForm calcSize];
	[postalButton setToolTip:NSLocalizedString(@"PostalDictionary",@"")];
	[[prefectureForm titleAtIndex:0] setStringValue:NSLocalizedString(@"Prefecture",@"県")];
	[[[prefectureForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[prefectureForm calcSize];
	[prefectureKanaField setStringValue:NSLocalizedString(@"PrefectureKana",@"")];
	[[cityForm titleAtIndex:0] setStringValue:NSLocalizedString(@"City",@"市区町村")];
	[[[cityForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[cityForm calcSize];
	[cityKanaField setStringValue:NSLocalizedString(@"CityKana",@"")];
	[[townForm titleAtIndex:0] setStringValue:NSLocalizedString(@"Town",@"町域")];
	[[[townForm cellAtIndex:0] cell] setSendsActionOnEndEditing:NO];
	[townForm calcSize];
	[townKanaField setStringValue:NSLocalizedString(@"TownKana",@"")];
	[[addressForm titleAtIndex:0] setStringValue:NSLocalizedString(@"Address",@"地番")];
	[[[addressForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[addressForm calcSize];
	[[telNumberForm titleAtIndex:0] setStringValue:NSLocalizedString(@"TelNumber",@"電話番号")];
	[[[telNumberForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[telNumberForm calcSize];
	[[faxNumberForm titleAtIndex:0] setStringValue:NSLocalizedString(@"FaxNumber",@"FAX番号")];
	[[[faxNumberForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[faxNumberForm calcSize];
	[[phsNumberForm titleAtIndex:0] setStringValue:NSLocalizedString(@"PHSNumber",@"携帯番号")];
	[[[phsNumberForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[phsNumberForm calcSize];
	[[emailCombo cell] setSendsActionOnEndEditing:YES];
	[emailCombo setUsesDataSource:NO];
	[[emailCombo cell] setFont:[NSFont fontWithName:NSLocalizedString(@"ScreenFont",@"") size:12.0]];
	[emailCombo setToolTip:NSLocalizedString(@"Item will be deleted with Delete command in Mainmenu.",@"項目はメインメニューの「削除」コマンドで削除されます。")];
	[emailTitleField setStringValue:NSLocalizedString(@"Email",@"Email")];
	[[urlCombo cell] setSendsActionOnEndEditing:YES];
	[urlCombo setUsesDataSource:NO];
	[[urlCombo cell] setFont:[NSFont fontWithName:NSLocalizedString(@"ScreenFont",@"") size:12.0]];
	[urlCombo setToolTip:NSLocalizedString(@"Item will be deleted with Delete command in Mainmenu.",@"項目はメインメニューの「削除」コマンドで削除されます。")];
	[urlTitleField setStringValue:NSLocalizedString(@"URLName",@"URL")];
	[[typeForm titleAtIndex:0] setStringValue:NSLocalizedString(@"Type",@"分類")];
	[[[typeForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[typeForm calcSize];
	[[birthDayForm titleAtIndex:0] setStringValue:NSLocalizedString(@"BirthDay",@"誕生日")];
	[[[birthDayForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[birthDayForm calcSize];
	[[birthDayForm cellAtIndex:0] setStringValue:@""];
	[[othersForm titleAtIndex:0] setStringValue:NSLocalizedString(@"Others",@"その他の情報")];
	[[[othersForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[othersForm calcSize];
	[[locateXForm titleAtIndex:0] setStringValue:NSLocalizedString(@"LocateX(D.MS)",@"")];
	[[locateYForm titleAtIndex:0] setStringValue:NSLocalizedString(@"LocateY(D.MS)",@"")];
	[[locateZForm titleAtIndex:0] setStringValue:NSLocalizedString(@"LocateZ(m)",@"")];
	[locateXForm calcSize];
	[locateYForm calcSize];
	[locateZForm calcSize];
	[[[locateXForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[[[locateYForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	[[[locateZForm cellAtIndex:0] cell] setSendsActionOnEndEditing:YES];
	NSNumberFormatter *dmsFormatter = [[NSNumberFormatter alloc] init];
	[dmsFormatter setFormatterBehavior:NSNumberFormatterBehavior10_4];
	[dmsFormatter setFormat:@"#,##0.00000000;0.00000000;-#,##0.00000000"];
	[[locateXForm cellAtIndex:0] setFormatter:dmsFormatter];
	[[locateYForm cellAtIndex:0] setFormatter:dmsFormatter];
	[[locateZForm cellAtIndex:0] setFormatter:dmsFormatter];
	[dmsFormatter release];
	[[locateXForm cellAtIndex:0] setStringValue:@""];
	[[locateYForm cellAtIndex:0] setStringValue:@""];
	[[locateZForm cellAtIndex:0] setStringValue:@""];
	[locateButton setToolTip:NSLocalizedString(@"Show map",@"")];
	[deleteButton setEnabled:NO];
	[zipPopUp setEnabled:NO];
	[postalDrawerTitleField setStringValue:NSLocalizedString(@"PostalDictionary",@"郵便番号辞書")];
	[zipDicTitleField setStringValue:NSLocalizedString(@"DictionaryName",@"辞書名")];
	[setPrefectureButton setTitle:NSLocalizedString(@"Set...",@"設定...")];
	[prefecSetButton setTitle:NSLocalizedString(@"Set",@"設定")];
	[prefecCancelButton setTitle:NSLocalizedString(@"Cancel",@"")];
	[prefecProgress setDisplayedWhenStopped:NO];
	[companyButton setTitle:NSLocalizedString(@"SearchWithCompany",@"事業所名による検索")];
	[zipDicField setStringValue:@""];
	[zipButton setState:NSControlStateValueOff];
	[pictureView setCompositingOperation:NSCompositingOperationSourceOver];
	[pictureView setImageFrameStyle:NSImageFramePhoto];
	[pictureView setImageScaling:NSImageScaleProportionallyUpOrDown];
	[pictureView setValue:[NSNumber numberWithBool:YES] forKey:ASImageViewDataRetainedKey];
	[pictureView setToolTip:NSLocalizedString(@"Drag fileicon of image here.",@"画像ファイルアイコンをここへドラッグ")];
	[imageDeleteButton setTitle:NSLocalizedString(@"ImageDelete",@"")];
	[[changeIndexForm titleAtIndex:0] setStringValue:NSLocalizedString(@"OldIndex",@"変更前のコード")];
	[[changeIndexForm titleAtIndex:1] setStringValue:NSLocalizedString(@"NewIndex",@"変更後のコード")];
	[changeIndexForm calcSize];
	[changeOKButton setTitle:NSLocalizedString(@"ChangeIndex",@"コード変更")];
	[deleteButton setTitle:NSLocalizedString(@"Delete",@"削除")];
	[parStyle setParagraphStyle:[NSParagraphStyle defaultParagraphStyle]];
	[parStyle setAlignment:NSTextAlignmentCenter];
	attributedTitle = [[NSAttributedString alloc] initWithString:NSLocalizedString(@"Enroll",@"登録") attributes:[NSDictionary dictionaryWithObjectsAndKeys:[NSFont fontWithName:NSLocalizedString(@"ScreenFont",@"") size:30.0],NSFontAttributeName,[NSColor colorWithCalibratedRed:0.0 green:0.28 blue:0.0 alpha:1.0],NSForegroundColorAttributeName,parStyle,NSParagraphStyleAttributeName,nil]];
	[enrollButton setAttributedTitle:attributedTitle];
	[parStyle release];
	[attributedTitle release];
	[[groupnameForm titleAtIndex:0] setStringValue:NSLocalizedString(@"GroupName",@"")];
	[groupnameForm calcSize];
	[addressbookCancelButton setTitle:NSLocalizedString(@"Cancel",@"")];
	[addressbookOKButton setTitle:NSLocalizedString(@"Save",@"")];
	[picturePathControl setURL:nil];
	[self setWindowFrameAutosaveName:NSStringFromClass([self class])];
	[[self window] setDelegate:self];
}

- (void)windowWillClose:(NSNotification *)notification {
	[[NSNotificationCenter defaultCenter] postNotificationName:NSWindowWillCloseNotification object:self];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[self window] setDelegate:nil];
	// [self close];または[[self document] close]; はやる必要が無い。 //
	// [self autorelease]; はやってはいけない。何故ならMainDocumentがdeallocを呼び出すからである。 //

}
/* - (void)_document:(NSDocument *)document shouldClose:(BOOL)shouldClose  contextInfo:(void  *)contextInfo { // [NSApp terminate]のときは呼ばれない。 //
// MainDocumentの方で、- (BOOL)shouldCloseWindowController:(NSWindowController *)windowControllerがYESを返すように設定してあるので、saveDocument:(id)senderまでの筋道はここでやらなければならない。これは、MainWindowControllerの他にもNSWindowControllerがあるのでやむを得ないことである。 //
	if (YES == [[self document] isDocumentEdited])
		NSBeginAlertSheet([NSString stringWithFormat:NSLocalizedString(@"Do you want to save the changes you made to the document \"%@\"?",@"閉じる前に、%@の変更内容を保存しますか?"),[[self document] displayName]],NSLocalizedString(@"Save",@""),NSLocalizedString(@"Don't Save",@""),NSLocalizedString(@"Cancel",@""),[self window],self,NULL,@selector(saveSheetDidEnd:returnCode:contextInfo:),NULL,NSLocalizedString(@"Your changes will be lost if you don't save them.",@"保存しないと、変更内容は失われます。"),nil);
	else
		[self close];
}
- (void)saveSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo

// closeではNSWindowWillCloseNotificationの通知が発せられない。performCloseの場合は、windowShouldCloseに戻ってからNSWindowWillCloseNotificationの通知が発せられる。 //
{
	switch (returnCode) {
	case NSAlertOtherReturn: // Cancel //
		break;
	case NSAlertAlternateReturn: // "Don't Save" //
		[[[self document] undoManager] removeAllActions];
		[[self document] updateChangeCount:NSChangeCleared];
		[self close];
        break;
	default: // Save //
		if (nil != [[self document] fileURL]) {
			[[self document] saveDocument:self];
			[self close];
		}
		else
			[[self document] saveDocument:self]; // [self close]; してはいけない。新規ドキュメントのときのsavePanelが消えてしまう。savePanelを必要とするのは新規ドキュメントの場合だけである。 //
		break;
    }
}

- (IBAction)saveDocument:(id)sender {
	[[self document] saveDocument:sender];
} */

- (void)populateForms:(NSDictionary *)aModel atIndex:(NSString *)index {
	NSString *value;
	NSArray *arrayValue;
	NSNumber *numberValue;
	NSString *empty = @"";

	[[rootIndexForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:RootIndexKey])) ? value : empty];
	[nameCombo setObjectValue:(nil != (value = [aModel objectForKey:NameKey])) ? value : empty];
	[nameKanaField setStringValue:(nil != (value = [aModel objectForKey:NameKanaKey])) ? value : empty];
	[[postForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:PostKey])) ? value : empty];
	[countryCombo setObjectValue:(nil != (value = [aModel objectForKey:CountryKey])) ? value : NSLocalizedString(@"LocalCountry",@"")];
	[[zipForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:ZipKey])) ? value : empty];
	[[prefectureForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:PrefectureKey])) ? value : empty];
	[[cityForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:CityKey])) ? value : empty];
	[[townForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:TownKey])) ? value : empty];
	[[addressForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:AddressKey])) ? value : empty];
	[prefectureKanaField setStringValue:(nil != (value = [aModel objectForKey:PrefectureKanaKey])) ? value : empty];
	[cityKanaField setStringValue:(nil != (value = [aModel objectForKey:CityKanaKey])) ? value : empty];
	[townKanaField setStringValue:(nil != (value = [aModel objectForKey:TownKanaKey])) ? value : empty];
	[[telNumberForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:TelNumberKey])) ? value : empty];
	[[faxNumberForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:FaxNumberKey])) ? value : empty];
	[[phsNumberForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:PHSNumberKey])) ? value : empty];
	[emailCombo removeAllItems];
	if ((nil != (arrayValue = [aModel objectForKey:EmailKey])) && (0 < [arrayValue count])) {
		[emailCombo addItemsWithObjectValues:arrayValue];
		[emailCombo selectItemAtIndex:0];
		[emailCombo setObjectValue:[emailCombo objectValueOfSelectedItem]];
	}
	else
		[emailCombo setObjectValue:empty];
	[urlCombo removeAllItems];
	if ((nil != (arrayValue = [aModel objectForKey:URLNameKey])) && (0 < [arrayValue count])) {
		[urlCombo addItemsWithObjectValues:arrayValue];
		[urlCombo selectItemAtIndex:0];
		[urlCombo setObjectValue:[urlCombo objectValueOfSelectedItem]];
	}
	else
		[urlCombo setObjectValue:empty];
	[[typeForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:TypeKey])) ? value : empty];
	[[birthDayForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:BirthDayKey])) ? value : empty];
	[[othersForm cellAtIndex:0] setStringValue:(nil != (value = [aModel objectForKey:OthersKey])) ? value : empty];
	if ((nil != (numberValue = [aModel objectForKey:LocateXKey])) && ([numberValue doubleValue] + 100.0 < 2.0 * RADIUSOFGRS80))
		[[locateXForm cellAtIndex:0] setDoubleValue:[numberValue doubleValue]];
	else
		[[locateXForm cellAtIndex:0] setStringValue:empty];
	if ((nil != (numberValue = [aModel objectForKey:LocateYKey])) && ([numberValue doubleValue] + 100.0 < 2.0 * RADIUSOFGRS80))
		[[locateYForm cellAtIndex:0] setDoubleValue:[numberValue doubleValue]];
	else
		[[locateYForm cellAtIndex:0] setStringValue:empty];
	if ((nil != (numberValue = [aModel objectForKey:LocateZKey])) && ([numberValue doubleValue] + 100.0 < 2.0 * RADIUSOFGRS80))
		[[locateZForm cellAtIndex:0] setDoubleValue:[numberValue doubleValue]];
	else
		[[locateZForm cellAtIndex:0] setStringValue:empty];
	NSFileWrapper *imagesWrapper = [[self document] valueForKey:EnrollImagesWrapperKey];
	NSData *imageData = nil;
	NSString *uti = nil;
	if (nil != imagesWrapper) {
		NSFileWrapper *imageWrapper;
		NSDictionary *wrappers = [imagesWrapper fileWrappers];
		for (NSString *imageFilename in wrappers) {
			if (YES == [[imageFilename stringByDeletingPathExtension] isEqualToString:index]) {
				imageWrapper = [wrappers objectForKey:imageFilename];
				if (nil != (imageData = [imageWrapper regularFileContents])) {
					if ((YES == [[[imageFilename pathExtension] lowercaseString] isEqualToString:@"svg"]) || (YES == [[[imageFilename pathExtension] lowercaseString] isEqualToString:@"svgz"]))
						uti = @"public.svg-image";
					else {
						CFStringRef utiRef = UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, (CFStringRef)[imageFilename pathExtension], NULL); // 拡張子からUTIを取得する。基準ファイルはimagesWrapperなのでNSWorkspaceのtypeOfFile:error:は使えない。SVGは暴走する。NULLにもならないでここで暴走する。 //
						if ((NULL == utiRef) || (0 == CFStringGetLength(utiRef)))
							NSLog(@"utiRef is NULL.");
						else {
							uti = (NSString *)utiRef;
							CFRelease(utiRef);
						}
					}
				}
				break;
			}
		}
	}
	if (nil != imageData) {
		if (nil == uti)
			NSLog(@"UTI is nil.");
		[pictureView setImageWithData:imageData ofType:uti];  // ASImageViewのcurrentURLはnil //
	}
	else
		[pictureView setImageWithData:nil ofType:nil];
	NSFileWrapper *soundsWrapper = [[self document] valueForKey:EnrollSoundsWrapperKey];
	NSData *soundData = nil;
	if (nil != soundsWrapper) {
		NSFileWrapper *soundWrapper;
		NSDictionary *wrappers = [soundsWrapper fileWrappers];
		for (NSString *soundFilename in wrappers) {
			if (YES == [[soundFilename stringByDeletingPathExtension] isEqualToString:index]) {
				soundWrapper = [wrappers objectForKey:soundFilename];
				soundData = (nil != soundWrapper) ? [soundWrapper regularFileContents] : nil;
				break;
			}
		}
	}
	if (nil != soundData) {
		if (nil == soundController) {
			soundController = [[ASSoundController alloc] init];
			[soundController setDocument:[self document]];
		}
		[soundController showWindow:self];
		[soundController setSoundData:soundData]; // soundControllerのURLはnil //
	}
	else {
		if (nil != soundController)
			[soundController setSoundData:nil];	
	}
	[[NSNotificationCenter defaultCenter] postNotificationName:IndexDidChangeNotification object:self userInfo:aModel];
	[[NSNotificationCenter defaultCenter] postNotificationName:MailToDidChangeNotification object:self userInfo:[NSDictionary dictionaryWithObject:[emailCombo objectValue] forKey:EmailKey]];
}

- (IBAction)searchWithIndex:(id)sender {
	NSString *index = strntrim([sender stringValue],0,NO);
	NSDictionary *enrollDictionary = [[self document] enrollDictionary];

	[sender setObjectValue:index];
	if (nil != enrollDictionary) {
		NSDictionary *aModel = [enrollDictionary objectForKey:index];

		if (nil != aModel) {
			[self populateForms:aModel atIndex:index];
			[[changeIndexForm cellAtIndex:0] setStringValue:index];
			[deleteButton setEnabled:YES];
			[nameCombo selectText:sender];
		}
		else {
			[self populateForms:[NSDictionary dictionary] atIndex:index];
			[[changeIndexForm cellAtIndex:0] setStringValue:@""];
			[deleteButton setEnabled:NO];
            NSAlert *alert = [[NSAlert alloc] init];
            [alert addButtonWithTitle:NSLocalizedString(@"Add",@"新規登録")];
            [alert addButtonWithTitle:NSLocalizedString(@"Cancel",@"取消し")];
            [alert setMessageText:NSLocalizedString(@"NotRegist",@"未登録")];
            [alert setInformativeText:index];
            [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
            [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
                if (NSAlertFirstButtonReturn == result)
                    [nameCombo selectText:nil];
                else
                    [indexCombo selectText:nil];
            }];
            [alert release];
			/* NSBeginAlertSheet(NSLocalizedString(@"NotRegist",@"未登録"),NSLocalizedString(@"Add",@"新規登録"),NSLocalizedString(@"Cancel",@"取消し"),nil,[self window],self,NULL,@selector(notRegistSheetDidEnd:returnCode:contextInfo:),NULL,index,nil); */
		}
	}
	else {
		[self populateForms:[NSDictionary dictionary] atIndex:index];
		[[changeIndexForm cellAtIndex:0] setStringValue:@""];
		[deleteButton setEnabled:NO];
		[nameCombo selectText:sender];
	}
}

- (IBAction)searchWithName:(id)sender {
	NSString *name = strntrim([nameCombo objectValue],0,NO);
	NSArray *searchResult = [[self document] modelOfName:name];

	if (nil != searchResult) {
		NSString *index = [searchResult objectAtIndex:0];
		NSDictionary *aModel = [searchResult objectAtIndex:1];

		[self populateForms:aModel atIndex:index];
		[indexCombo setObjectValue:index];
		[nameCombo setObjectValue:name];
	}
}

- (IBAction)setDate:(id)sender {
	if (NO == [[sender stringValue] isEqualToString:@""]) {
		NSString *newDate = fldate([sender stringValue]);

		if (nil != newDate)
			[sender setStringValue:newDate];
		else
			[sender selectText:sender];
	}
}

- (void)makeZipPopUp:(PostalFind *)postal finds:(NSArray *)anArray {
	NSUInteger i;
	NSMutableString *popString;
	NSString *obj;
	NSDictionary *topDic = [anArray objectAtIndex:0];
	NSString *empty = @"";

	[zipPopUp removeAllItems];
	[zipPopUp setEnabled:YES];
	[zipButton setState:(1 < [anArray count]) ? NSControlStateValueOn : NSControlStateValueOff];
	for (i = 0; (i < [anArray count]) && (i < 30); i++) {
		popString = [NSMutableString stringWithString:[[anArray objectAtIndex:i] objectForKey:PostalDictionaryProtocolFormKey]];
		if (YES == [[[anArray objectAtIndex:i] objectForKey:PostalDictionaryProtocolTypeKey] isEqualToString:PostalDictionaryProtocolTownValue])
			[popString appendFormat:@" %@",[[anArray objectAtIndex:i] objectForKey:PostalDictionaryProtocolTownKey]];
		else
			[popString appendFormat:@" %@",[[anArray objectAtIndex:i] objectForKey:PostalDictionaryProtocolCompanyKey]];
		[zipPopUp addItemWithTitle:popString];
	}
    if (i >= 30) {
        NSAlert *alert = [[NSAlert alloc] init];
        // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
        [alert setMessageText:NSLocalizedString(@"PopUpButton too many items.",@"ポップアップボタンに入りきりません。検索範囲を絞ってください。")];
        [alert setInformativeText:[NSString string]];
        [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
        [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
            // 何もやることがない //
        }];
        [alert release];
		/* NSBeginAlertSheet(NSLocalizedString(@"PopUpButton too many items.",@"ポップアップボタンに入りきりません。検索範囲を絞ってください。"),nil,nil,nil,[self window],self,NULL,NULL,NULL,[NSString string],nil); */
    }
	if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolFormKey])) obj = empty;
	[[zipForm cellAtIndex:0] setStringValue:obj];
	if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolPrefectureKey])) obj = empty;
	[[prefectureForm cellAtIndex:0] setStringValue:obj];
	if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolCityKey])) obj = empty;
	[[cityForm cellAtIndex:0] setStringValue:obj];
	if (YES == [[topDic objectForKey:PostalDictionaryProtocolTypeKey] isEqualToString:PostalDictionaryProtocolTownValue]) {
		if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolPrefectureKanaKey])) obj = empty;
		[prefectureKanaField setStringValue:obj];
		if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolCityKanaKey])) obj = empty;
		[cityKanaField setStringValue:obj];
		if (YES == [[topDic objectForKey:PostalDictionaryProtocolTownKey] isEqualToString:[postal otherTown]]) {
			[[townForm cellAtIndex:0] setStringValue:empty];
			[townKanaField setStringValue:empty];
		}
		else {
			if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolTownKey])) obj = empty;
			[[townForm cellAtIndex:0] setStringValue:obj];
			if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolTownKanaKey])) obj = empty;
			[townKanaField setStringValue:obj];
		}
	}
	else {
		if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolCompanyKey])) obj = empty;
		[nameCombo setObjectValue:obj];
		if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolCompanyKanaKey])) obj = empty;
		[nameKanaField setStringValue:[NSString stringWithString:obj]];
		[prefectureKanaField setStringValue:empty];
		[cityKanaField setStringValue:empty];
		if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolTownKey])) obj = empty;
		[[townForm cellAtIndex:0] setStringValue:obj];
		[townKanaField setStringValue:empty];
		if (nil == (obj = [topDic objectForKey:PostalDictionaryProtocolAddressKey])) obj = empty;
		[[addressForm cellAtIndex:0] setStringValue:obj];
	}
	[townForm selectTextAtIndex:0];
}
- (IBAction)setZip:(id)sender {
	if (YES == self.postalPopover.isShown) {
		PostalFind *postal = [PostalFind postalFind];

		if (YES == [postal postalSet]) {
			NSArray *cities;

			[postal setDelegate:self];
			[zipDicField setStringValue:[postal dictionaryName]];
			if ((nil != (cities = [postal findWithZip:[sender stringValue]])) && (0 < [cities count]))
				[self makeZipPopUp:postal finds:cities];
			else
				[sender selectText:nil];
			[postal setDelegate:nil];
		}
		else {
			[self.postalPopover close];
			[prefectureForm selectTextAtIndex:0];
		}
	}
	else
		[prefectureForm selectTextAtIndex:0];
}
- (IBAction)setTown:(id)sender {
	if (YES == self.postalPopover.isShown) {
		PostalFind *postal = [PostalFind postalFind];

		if (YES == [postal postalSet]) {
			NSArray *numbers;

			[postal setDelegate:self];
			[zipDicField setStringValue:[postal dictionaryName]];
			if ((nil != (numbers = [postal findWithPrefecture:[[prefectureForm cellAtIndex:0] stringValue] City:[[cityForm cellAtIndex:0] stringValue] Town:[[townForm cellAtIndex:0] stringValue]])) && (0 < [numbers count]))
				[self makeZipPopUp:postal finds:numbers];
			else
				[sender selectText:nil];
			[postal setDelegate:nil];
		}
		else {
			[self.postalPopover close];
			[addressForm selectTextAtIndex:0];
		}
	}
	else
		[addressForm selectTextAtIndex:0];
}

- (IBAction)toggleChangeIndex:(id)sender {
	if (NO == self.changeIndexPopover.isShown) {
		[self.changeIndexPopover showRelativeToRect:postForm.bounds ofView:postForm preferredEdge:NSMinXEdge];
	}
	else
		[self.changeIndexPopover close];
}
- (IBAction)indexChange:(id)sender {
	NSString *oldIndex = strntrim([[changeIndexForm cellAtIndex:0] stringValue],0,NO);
	NSString *newIndex = strntrim([[changeIndexForm cellAtIndex:1] stringValue],0,NO);

	if ((nil != oldIndex) && (NO == [oldIndex isEqualToString:@""]) && (nil != newIndex) && (NO == [newIndex isEqualToString:@""])) {
		NSDictionary *dictionary = [[self document] enrollDictionary];
		NSDictionary *newModel = [dictionary objectForKey:newIndex];
		NSDictionary *oldModel = [dictionary objectForKey:oldIndex];
		
		if ((nil != oldModel) && (nil == newModel)) {
			if (YES == [[self document] changeIndex:oldIndex to:newIndex]) {
				[[changeIndexForm cellAtIndex:0] setStringValue:newIndex];
				[[changeIndexForm cellAtIndex:1] setStringValue:@""];
				if (YES == [oldIndex isEqualToString:strntrim([indexCombo objectValue],0,NO)]) {
					[indexCombo setObjectValue:newIndex];
					dictionary = [[self document] enrollDictionary];
					[self populateForms:[dictionary objectForKey:newIndex] atIndex:newIndex];
				}
			}
		}
		else {
            NSAlert *alert = [[NSAlert alloc] init];
            // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
            [alert setMessageText:(nil == oldModel) ? NSLocalizedString(@"NotRegist","未登録") : NSLocalizedString(@"AlreadyRegistered","登録済で使用できません")];
            [alert setInformativeText:(nil == oldModel) ? oldIndex : newIndex];
            [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
            [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
                    // 何もやることがない //
            }];
            [alert release];
            /* NSBeginAlertSheet(NSLocalizedString(@"NotRegist","未登録"),nil,nil,nil,[self window],self,NULL,NULL,NULL,oldIndex,nil);
             else
				NSBeginAlertSheet(NSLocalizedString(@"AlreadyRegistered","登録済で使用できません"),nil,nil,nil,[self window],self,NULL,NULL,NULL,newIndex,nil); */
		}
	}
}

- (void)deleteModel:(id)sender {
	NSString *oldIndex = strntrim([indexCombo objectValue],0,NO);

    if ((nil != oldIndex) && (NO == [oldIndex isEqualToString:@""]) && (nil != [[[self document] enrollDictionary] objectForKey:oldIndex])) {
        NSAlert *alert = [[NSAlert alloc] init];
        [alert addButtonWithTitle:NSLocalizedString(@"Cancel",@"")];
        [alert addButtonWithTitle:NSLocalizedString(@"Delete",@"")];
        [alert setMessageText:[NSString stringWithFormat:NSLocalizedString(@"Do you really want to delete \"%@\"",@""),oldIndex]];
        [alert setInformativeText:[NSString string]];
        [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
        [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
            if (NSAlertSecondButtonReturn == result) {
                NSString *oldIndex = strntrim([indexCombo objectValue],0,NO);
                
                if (YES == [[self document] deleteIndex:oldIndex]) {
                    if (YES == [oldIndex isEqualToString:strntrim([indexCombo objectValue],0,NO)]) {
                        [indexCombo setObjectValue:@""];
                        [self populateForms:[NSDictionary dictionary] atIndex:oldIndex];
                    }
                }
            }
        }];
        [alert release];
		/* NSBeginAlertSheet([NSString stringWithFormat:NSLocalizedString(@"Do you really want to delete \"%@\"",@""),oldIndex],NSLocalizedString(@"Cancel",@""),NSLocalizedString(@"Delete",@""),nil,[self window],self,NULL,@selector(deleteSheetDidEnd:returnCode:contextInfo:),NULL,[NSString string],nil); */
    }
}

- (NSMutableDictionary *)makeModel {
	NSString *stringValue;
	NSArray *arrayValue;
	NSMutableArray *setArrayValue;
	double numberValue;
	NSMutableDictionary *aModel = [NSMutableDictionary dictionary];

	[self setEmail:emailCombo];
	[self setURL:urlCombo];
	if ((nil != (stringValue = strntrim([[rootIndexForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:RootIndexKey];
		[aModel setObject:stringValue forKey:NameKey];
	if ((nil != (stringValue = strntrim([nameCombo objectValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:NameKey];
	if ((nil != (stringValue = strntrim([nameKanaField stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:NameKanaKey];
	if ((nil != (stringValue = strntrim([[postForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:PostKey];
	if ((nil != (stringValue = strntrim([countryCombo objectValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:CountryKey];
	if ((nil != (stringValue = strntrim([[zipForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:ZipKey];
	if ((nil != (stringValue = strntrim([[prefectureForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:PrefectureKey];
	if ((nil != (stringValue = strntrim([prefectureKanaField stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:PrefectureKanaKey];
	if ((nil != (stringValue = strntrim([[cityForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:CityKey];
	if ((nil != (stringValue = strntrim([cityKanaField stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:CityKanaKey];
	if ((nil != (stringValue = strntrim([[townForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:TownKey];
	if ((nil != (stringValue = strntrim([townKanaField stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:TownKanaKey];
	if ((nil != (stringValue = strntrim([[addressForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:AddressKey];
	if ((nil != (stringValue = strntrim([[telNumberForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:TelNumberKey];
	if ((nil != (stringValue = strntrim([[faxNumberForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:FaxNumberKey];
	if ((nil != (stringValue = strntrim([[phsNumberForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:PHSNumberKey];
	arrayValue = [emailCombo objectValues];
	setArrayValue = [NSMutableArray array];
    for (NSString *aStr in arrayValue) {
        if (NO == [aStr isEqualToString:@""])
            [setArrayValue addObject:aStr];
    }
	if (0 < [setArrayValue count])
		[aModel setObject:setArrayValue forKey:EmailKey];
	arrayValue = [urlCombo objectValues];
    setArrayValue = [NSMutableArray array];
    for (NSString *aStr in arrayValue) {
        if (NO == [aStr isEqualToString:@""])
            [setArrayValue addObject:aStr];
    }
	if (0 < [setArrayValue count])
		[aModel setObject:setArrayValue forKey:URLNameKey];
	if ((nil != (stringValue = strntrim([[typeForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:TypeKey];
	if ((nil != (stringValue = strntrim([[birthDayForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:BirthDayKey];
	if ((nil != (stringValue = strntrim([[othersForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:stringValue forKey:OthersKey];
	if ((nil != (stringValue = strntrim([[locateXForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""])) {
		numberValue = [[locateXForm cellAtIndex:0] doubleValue];
		if (numberValue + 100.0 < 2.0 * RADIUSOFGRS80)
			[aModel setObject:[NSNumber numberWithDouble:numberValue] forKey:LocateXKey];
	}
	if ((nil != (stringValue = strntrim([[locateYForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""])) {
		numberValue = [[locateYForm cellAtIndex:0] doubleValue];
		if (numberValue + 100.0 < 2.0 * RADIUSOFGRS80)
			[aModel setObject:[NSNumber numberWithDouble:numberValue] forKey:LocateYKey];
	}
	if ((nil != (stringValue = strntrim([[locateZForm cellAtIndex:0] stringValue],0,NO))) && (NO == [stringValue isEqualToString:@""])) {
		numberValue = [[locateZForm cellAtIndex:0] doubleValue];
		if (numberValue + 100.0 < 2.0 * RADIUSOFGRS80)
			[aModel setObject:[NSNumber numberWithDouble:numberValue] forKey:LocateZKey];
	}
	/* if ((nil != (stringValue = [pictureField stringValue])) && (NO == [stringValue isEqualToString:@""]))
		[aModel setObject:[pictureField stringValue] forKey:PictureFilenameKey];
	if ((nil != soundController) && (nil != (urlValue = [soundController url]))) {
		[aModel setObject:[urlValue path] forKey:SoundFilenameKey];
		NSLog(@"setSound");
	} */
	return aModel;
}

- (IBAction)enroll:(id)sender {
	NSString *index = strntrim([indexCombo objectValue],0,NO);

	if ((nil != index) && (NO == [index isEqualToString:@""])) {
		// pictureに変更ないときと、もともと無いときと削除されたときはURLがnil。soundに変更がないときと、もともと無いときと削除されたときはURLがnil。 //
		if (YES == [[self document] addModel:[self makeModel] picture:[picturePathControl URL] pictureData:[pictureView valueForKey:ASImageViewImageDataKey] sound:(nil != soundController) ? [soundController url] : nil soundData:(nil != soundController) ? [soundController soundData] : nil forKey:index])
			[deleteButton setEnabled:YES];
		[indexCombo setNumberOfVisibleItems:(20 > [[[self document] indexes] count]) ? [[[self document] indexes] count] : 20];
		[nameCombo setNumberOfVisibleItems:(20 > [[[self document] indexes] count]) ? [[[self document] indexes] count] : 20];
	}
}

- (IBAction)hagaki:(id)sender {
	NSDictionary *enroll = [[self document] enrollDictionary];

	if ((nil != enroll) && (0 < [enroll count])) {
		NSDictionary *aModel;
		HagakiPrint *newInstance = [HagakiPrint hagakiPrint];

		[newInstance setDocument:[self document]];
		[newInstance showWindow:sender];
		if (nil != (aModel = [enroll objectForKey:[indexCombo objectValue]]))
			[[NSNotificationCenter defaultCenter] postNotificationName:IndexDidChangeNotification object:self userInfo:aModel];
	}
    else {
        NSAlert *alert = [[NSAlert alloc] init];
        // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
        [alert setMessageText:[[self document] displayName]];
        [alert setInformativeText:NSLocalizedString(@"Enroll is empty.",@"名簿データが存在しません。")];
        [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
        [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
            // 何もやることがない //
        }];
        [alert release];
		/* NSBeginInformationalAlertSheet([[self document] displayName],nil,nil,nil,[self window],self,NULL,NULL,NULL,NSLocalizedString(@"Enroll is empty.",@"名簿データが存在しません。"),nil); */
    }
}
- (IBAction)report:(id)sender {
	NSDictionary *enroll = [[self document] enrollDictionary];

	if ((nil != enroll) && (0 < [enroll count])) {
		Report *newInstance = [Report report];

		[newInstance setDocument:[self document]];
		[newInstance showWindow:sender];
	}
    else {
        NSAlert *alert = [[NSAlert alloc] init];
        // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
        [alert setMessageText:[[self document] displayName]];
        [alert setInformativeText:NSLocalizedString(@"Enroll is empty.",@"名簿データが存在しません。")];
        [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
        [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
            // 何もやることがない //
        }];
        [alert release];
		/* NSBeginInformationalAlertSheet([[self document] displayName],nil,nil,nil,[self window],self,NULL,NULL,NULL,NSLocalizedString(@"Enroll is empty.",@"名簿データが存在しません。"),nil); */
    }
}
- (IBAction)sound:(id)sender {
	NSDictionary *enroll = [[self document] enrollDictionary];
	NSString *index = [indexCombo objectValue];

	if ((nil != enroll) && (0 < [enroll count])) {
		NSFileWrapper *soundsWrapper = [[self document] valueForKey:EnrollSoundsWrapperKey];
		NSData *soundData = nil;
		if (nil != soundsWrapper) {
			NSFileWrapper *soundWrapper;
			NSDictionary *wrappers = [soundsWrapper fileWrappers];
			for (NSString *soundFilename in wrappers) {
				if (YES == [[soundFilename stringByDeletingPathExtension] isEqualToString:index]) {
					soundWrapper = [wrappers objectForKey:soundFilename];
					soundData = (nil != soundWrapper) ? [soundWrapper regularFileContents] : nil;
					break;
				}
			}
		}
		if (nil != soundData) {
			if (nil == soundController) {
				soundController = [[ASSoundController alloc] init];
				[soundController setDocument:[self document]];
			}
			[soundController showWindow:self];
			[soundController setSoundData:soundData]; // soundFileWellへはnilをセットするだけなので何もしなくて良い。 //
		}
		else {
			if (nil != soundController)
				[soundController setSoundData:nil];
		}
	}
    else {
        NSAlert *alert = [[NSAlert alloc] init];
        // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
        [alert setMessageText:[[self document] displayName]];
        [alert setInformativeText:NSLocalizedString(@"Enroll is empty.",@"名簿データが存在しません。")];
        [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
        [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
            // 何もやることがない //
        }];
        [alert release];
		/* NSBeginInformationalAlertSheet([[self document] displayName],nil,nil,nil,[self window],self,NULL,NULL,NULL,NSLocalizedString(@"Enroll is empty.",@"名簿データが存在しません。"),nil); */
    }
}

- (IBAction)sendEmail:(id)sender {
	NSString *tempString = strntrim([emailCombo objectValue],0,NO);

	if ((nil != tempString) && (NO == [tempString isEqualToString:@""])) {
		NSString *urlString = nil;
		NSScanner *aScanner = [NSScanner scannerWithString:tempString];
		NSMutableCharacterSet *skipSet = [[NSCharacterSet whitespaceAndNewlineCharacterSet] mutableCopy];

		[skipSet addCharactersInString:@","];
		if ((YES == [aScanner scanUpToCharactersFromSet:skipSet intoString:&urlString]) && (nil != urlString)) {
			NSURL *url = [NSURL URLWithString:[NSString stringWithFormat:@"mailto:%@",urlString]];

			if (nil != url)
				[[NSWorkspace sharedWorkspace] openURL:url];
			/* if (NO == [[NSWorkspace sharedWorkspace] openURL:url]) {
				MailSend *newInstance = [MailSend mailSend];
				NSDictionary *selfDic = [[self document] selfDictionary];

				[newInstance showWindow:sender];
				[newInstance setDocument:[self document]];
				[newInstance setTo:urlString];
				[newInstance setFrom:(nil != selfDic) ? [selfDic objectForKey:EmailKey] : nil selectedEmail:[emailCombo stringValue]];
			} */
		}
		[skipSet release];
	}
}
- (IBAction)setEmail:(id)sender {
	NSString *value = strntrim([sender stringValue],0,NO);

	if ((nil != value) && (NO == [value isEqualToString:@""])) {
		NSArray *objectValues = [sender objectValues];

		[[NSNotificationCenter defaultCenter] postNotificationName:MailToDidChangeNotification object:self userInfo:[NSDictionary dictionaryWithObject:value forKey:EmailKey]];
		if (NO == [objectValues containsObject:value])
			[sender addItemWithObjectValue:value];
	}
}
- (IBAction)goURL:(id)sender {
	NSString *tempString = strntrim([urlCombo objectValue],0,NO);

	if ((nil != tempString) && (NO == [tempString isEqualToString:@""])) {
		NSURL *url;
		NSString *urlString;
		NSScanner *aScanner = [NSScanner scannerWithString:tempString];
		NSMutableCharacterSet *skipSet = [[NSCharacterSet whitespaceAndNewlineCharacterSet] mutableCopy];

		[skipSet addCharactersInString:@","];
		if ((YES == [aScanner scanUpToCharactersFromSet:skipSet intoString:&urlString]) && (nil != (url = [NSURL URLWithString:urlString]))) {
            if (NO == [[NSWorkspace sharedWorkspace] openURL:url]) {
                NSAlert *alert = [[NSAlert alloc] init];
                // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
                [alert setMessageText:NSLocalizedString(@"URLString is not well-formed.",@"URL文字列の形式が間違っています。")];
                [alert setInformativeText:[NSString stringWithFormat:@"absolute:%@ scheme:%@ host:%@ path:%@",[url absoluteString],[url scheme],[url host],[url path]]];
                [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
                [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
                    // 何もやることがない //
                }];
                [alert release];
				/* NSBeginAlertSheet(NSLocalizedString(@"URLString is not well-formed.",@"URL文字列の形式が間違っています。"),nil,nil,nil,[self window],self,NULL,NULL,NULL,[NSString stringWithFormat:@"absolute:%@ scheme:%@ host:%@ path:%@",[url absoluteString],[url scheme],[url host],[url path]],nil); */
            }
		}
        else {
            NSAlert *alert = [[NSAlert alloc] init];
            // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
            [alert setMessageText:NSLocalizedString(@"URLString is not well-formed.",@"URL文字列の形式が間違っています。")];
            [alert setInformativeText:urlString];
            [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
            [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
                // 何もやることがない //
            }];
            [alert release];
			/* NSBeginAlertSheet(NSLocalizedString(@"URLString is not well-formed.",@"URL文字列の形式が間違っています。"),nil,nil,nil,[self window],self,NULL,NULL,NULL,urlString,nil); */
        }
		[skipSet release];
	}
}

- (IBAction)setURL:(id)sender {
	NSString *value = strntrim([sender stringValue],0,NO);

	if ((nil != value) && (NO == [value isEqualToString:@""])) {
		NSArray *objectValues = [sender objectValues];

		if (NO == [objectValues containsObject:value])
			[sender addItemWithObjectValue:value];
	}
}

- (void)clearURL:(NSComboBox *)sender {
	NSString *value = strntrim([sender objectValue],0,NO);
	NSArray *objectValues = [sender objectValues];

	if (YES == [objectValues containsObject:value]) {
		[sender removeItemWithObjectValue:value];
		[sender setObjectValue:@""];
	}
}

- (IBAction)clear:(id)sender {
	if (nil != [urlCombo currentEditor])
		[self clearURL:urlCombo];
	else {
		if (nil != [emailCombo currentEditor])
			[self clearURL:emailCombo];
		else {
			if (nil != [indexCombo currentEditor])
				[self deleteModel:indexCombo];
		}
	}
}

- (IBAction)postal:(id)sender {
	if (NO == self.postalPopover.isShown) {
		PostalFind *newInstance = [PostalFind postalFind];
		if (NO == [newInstance postalSet]) {
			NSAlert *alert = [[NSAlert alloc] init];
			// [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
			[alert setMessageText:NSLocalizedString(@"Could not Connect ZipDictionaryServer",@"郵便番号辞書サーバに接続できませんでした")];
			[alert setInformativeText:[NSString string]];
			[alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
			[alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
				// 何もやることがない //
			}];
			[alert release];
			/* NSBeginAlertSheet(NSLocalizedString(@"Could not Connect ZipDictionaryServer",@"郵便番号辞書サーバに接続できませんでした"),nil,nil,nil,[self window],self,NULL,NULL,NULL,[NSString string],nil); */
			[zipDicField setStringValue:@""];
		}
		else {
			[newInstance setDelegate:self];
			[zipDicField setStringValue:[newInstance dictionaryName]];
			[self.postalPopover showRelativeToRect:self.window.contentView.bounds ofView:self.window.contentView preferredEdge:NSMinYEdge];
			[newInstance setDelegate:nil];
		}
	}
	else
		[self.postalPopover close];
}

- (IBAction)company:(id)sender {
	PostalFind *postal = [PostalFind postalFind];

	if (YES == [postal postalSet]) {
		[zipDicField setStringValue:[postal dictionaryName]];
		if (NO == [[nameCombo objectValue] isEqualToString:@""]) {
			NSArray *numbers = [postal findWithCompany:[nameCombo objectValue]];

			if (0 < [numbers count])
				[self makeZipPopUp:postal finds:numbers];
			else
				[nameCombo selectText:sender];
		}
		else
			[nameCombo selectText:sender];
	}
	else
		[self.postalPopover close];
}
- (void)zipPopUped:(id)sender {
	PostalFind *postal = [PostalFind postalFind];

	if (YES == [postal postalSet]) {
		NSString *zipString = [sender titleOfSelectedItem];
		NSUInteger length = [zipString length];
		NSRange range = [zipString rangeOfString:@" "];
		NSString *zip = [zipString substringWithRange:NSMakeRange(0,range.location)];
		NSString *searchString = [zipString substringWithRange:NSMakeRange(range.location + 1,length - range.location - 1)];
	
		[zipDicField setStringValue:[postal dictionaryName]];
		if (NO == [searchString isEqualToString:@""]) {
			NSArray *towns = [postal findWithZip:zip];

			[[zipForm cellAtIndex:0] setStringValue:zip];
            for (NSDictionary *aDictionary in towns) {
                if (YES == [[aDictionary objectForKey:PostalDictionaryProtocolTypeKey] isEqualToString:PostalDictionaryProtocolTownValue]) {
                    if (YES == [[aDictionary objectForKey:PostalDictionaryProtocolTownKey] isEqualToString:searchString]) {
                        [[prefectureForm cellAtIndex:0] setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolPrefectureKey]];
                        [prefectureKanaField setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolPrefectureKanaKey]];
                        [[cityForm cellAtIndex:0] setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolCityKey]];
                        [cityKanaField setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolCityKanaKey]];
                        if (YES == [[aDictionary objectForKey:PostalDictionaryProtocolTownKey] isEqualToString:[postal otherTown]]) {
                            [[townForm cellAtIndex:0] setStringValue:@""];
                            [townKanaField setStringValue:@""];
                        }
                        else {
                            [[townForm cellAtIndex:0] setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolTownKey]];
                            [townKanaField setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolTownKanaKey]];
                        }
                        break;
                    }
                }
                else {
                    if (YES == [[aDictionary objectForKey:PostalDictionaryProtocolCompanyKey] isEqualToString:searchString]) {
                        [[prefectureForm cellAtIndex:0] setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolPrefectureKey]];
                        [prefectureKanaField setStringValue:@""];
                        [[cityForm cellAtIndex:0] setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolCityKey]];
                        [cityKanaField setStringValue:@""];
                        [[townForm cellAtIndex:0] setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolTownKey]];
                        [townKanaField setStringValue:@""];
                        [[addressForm cellAtIndex:0] setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolAddressKey]];
                        [nameCombo setObjectValue:[aDictionary objectForKey:PostalDictionaryProtocolCompanyKey]];
                        [nameKanaField setStringValue:[aDictionary objectForKey:PostalDictionaryProtocolCompanyKanaKey]];
                        break;
                    }
                }
            }
		}
	}
	else
		[self.postalPopover close];
}
- (IBAction)setPostalPrefec:(id)sender {
	PostalFind *postal = [PostalFind postalFind];

	if ((nil != postal) && (YES == [postal postalSet])) {
		NSInteger row,rs;
		NSInteger i,j,cs;
		NSButtonCell *prefecCell;
		NSDictionary *workingDic = [postal workingPrefectures];
		NSDictionary *enableDic = [postal enablePrefectures];
		NSArray *keyArray = [[enableDic allKeys] sortedArrayUsingSelector:@selector(compare:)];
		NSInteger k = (NSInteger)[keyArray count];

		[prefecMatrix getNumberOfRows:&rs columns:&cs];
		for (row = rs - 1; row >= 0; row--)
			[prefecMatrix removeRow:row];
		for (i = 0; i <= k / cs; i++) {
			if (k > i * cs)
				[prefecMatrix addRow];
			for (j = 0; (j < cs) && (k > i * cs + j); j++) {
				prefecCell = [prefecMatrix cellAtRow:i column:j];
				[prefecCell setTitle:[enableDic objectForKey:[keyArray objectAtIndex:(NSUInteger)(i * cs + j)]]];
				[prefecCell setEnabled:YES];
				if (nil != [workingDic objectForKey:[keyArray objectAtIndex:(NSUInteger)(i * cs + j)]])
					[prefecCell setState:NSControlStateValueOn];
				else
					[prefecCell setState:NSControlStateValueOff];
			}
		}
		if (0 < (k % cs)) {
			for (--i, j = k % cs; j < cs; j++) {
				prefecCell = [prefecMatrix cellAtRow:i column:j];
				[prefecCell setTitle:@""];
				[prefecCell setEnabled:NO];
				[prefecCell setState:NSControlStateValueOff];
			}
		}
		if (1 == [NSApp runModalForWindow:prefecWindow]) {
			NSMutableArray *prefecArray = [NSMutableArray array];

			[prefecProgress setIndeterminate:YES];
			[prefecProgress setUsesThreadedAnimation:YES];
			[prefecProgress startAnimation:sender];
			[prefecProgress display];
			for (i = 0; i <= k / cs; i++) {
				for (j = 0; (j < cs) && (k > i * cs + j); j++) {
					prefecCell = [prefecMatrix cellAtRow:i column:j];
					if (NSControlStateValueOn == [prefecCell state])
						[prefecArray addObject:[prefecCell title]];
				}
			}
			if (0 == [postal setPrefectures:prefecArray])
				[zipDicField setStringValue:@""];
			else
				[zipDicField setStringValue:[postal dictionaryName]];
			[prefecProgress stopAnimation:sender];
		}
		else
			[zipDicField setStringValue:@""];
		[prefecWindow orderOut:nil];
	}
	else {
		[zipDicField setStringValue:@""];
	}
}
- (IBAction)prefecSet:(id)sender {
	[NSApp stopModalWithCode:1];
}
- (IBAction)prefecCancel:(id)sender {
	[NSApp stopModalWithCode:0];
}
- (void)postalDidDie:(id)sender {
	[zipPopUp setEnabled:NO];
	[zipDicField setStringValue:@""];
	[zipButton setState:NSControlStateValueOff];
	[self.postalPopover close];
}
- (void)postalDidInitialize:(id)sender
{
}

// Delegate //
- (NSInteger)numberOfItemsInComboBox:(NSComboBox *)aComboBox {
	if (indexCombo == aComboBox)
		return (nil != [[self document] indexes]) ? (NSInteger)[[[self document] indexes] count] : 0;
	else {
		if (nameCombo == aComboBox)
			return (nil != [[self document] indexes]) ? (NSInteger)[[[self document] indexes] count] : 0;
		else
			return 0;
	}
}
- (id)comboBox:(NSComboBox *)aComboBox objectValueForItemAtIndex:(NSInteger)index {
	if (indexCombo == aComboBox) {
		NSParameterAssert(index >= 0 && index < (NSInteger)[[[[self document] enrollDictionary] allKeys] count]);
		return [[[self document] indexes] objectAtIndex:(NSUInteger)index];
	}
	else {
		if (nameCombo == aComboBox) {
			NSParameterAssert(index >= 0 && index < (NSUInteger)[[[[self document] enrollDictionary] allKeys] count]);
			return [[self document] nameAtNumber:(NSUInteger)index];
		}
		return nil;
	}
}
- (NSUInteger)comboBox:(NSComboBox *)aComboBox indexOfItemWithStringValue:(NSString *)string {
	// returns the index for the item that matches the specified string. If this method is available, the combo box performs incremental searches when the user types into the text field with the pop-up list displayed. //
	if (nil != string) {
		if (indexCombo == aComboBox)
			return [[[self document] indexes] indexOfObject:string];
		else {
			if (countryCombo == aComboBox)
				return [[countryCodeDic() allKeys] indexOfObject:string];
			else {
				if (nameCombo == aComboBox)
					return [[self document] numberOfName:string];
				else
					return NSNotFound;
			}
		}
	}
	else
		return NSNotFound;
}

// 以下の2つのメソッドは実装しない方が調子良い。 //
/* - (NSString *)firstHeadingMatchingPrefix:(NSString *)prefix object:(NSComboBox *)aComboBox
{
	NSEnumerator *stringEnum;
	NSString *string = nil;
	NSString *lowercasePrefix = [prefix lowercaseString];

	if (indexCombo == aComboBox)
		stringEnum = [[[self document] indexes] objectEnumerator];
	else {
		if (nameCombo == aComboBox)
			stringEnum = [[[self document] names] objectEnumerator];
		else
			stringEnum = nil;
	}
	if (nil != stringEnum) {
		while ((string = [stringEnum nextObject])) {
			if ([[string lowercaseString] hasPrefix: lowercasePrefix])
				break;
		}
	}
	return string;
}
- (NSString *)comboBox:(NSComboBox *)aComboBox completedString:(NSString *)string
// returns a string that begins with the specified string. If autocompletion is enabled, the combo box tries to complete what the user enters into the text field with an item from the pop-up list. If this method isn't available and autocompletion is enabled, the combo box goes through each item one-by-one to find a completion. //
{
	NSString *candidate = [self firstHeadingMatchingPrefix:string object:aComboBox];

	return (nil != candidate) ? candidate : string;
} */

- (IBAction)revertDocumentToSaved:(id)sender {
	[[self document] revertDocumentToSaved:sender];
	if (NO == [[indexCombo objectValue] isEqualToString:@""]) {
		NSString *index = strntrim([indexCombo objectValue],0,NO);
		[indexCombo setStringValue:@""];
		[self populateForms:[NSDictionary dictionary] atIndex:index];
	}
}

// modf() は処理系の違いにより戻り値の符号が異なるので絶対値に直してから処理をすること。 //
static double dtor(double x)
// DDD.MMSSSSSS表現のxをラジアンに変換 //
{
	double a,b,c;
	NSInteger sgn = 1;	
	double error = 1.0e-11; // modf()が1.0を返してしまうことを防ぐ為の数値。 //

	if (x < 0.0) {
		x = fabs(x);
		sgn = -1;
	}
	/* x = modf(modf(modf(x,&a) * 100.0 + error, &b) * 100.0 + error, &c) * 2.777777777777777777e-4; // 4.896618179206312657133e-15くらいの誤差を引きずってしまう。 //
	x += c * 2.777777777777777777e-4;
	x += a + b * 1.666666666666666666e-2;
	x *= 1.745329251994329547437e-2; */
	x = modf(modf(modf(x,&a) * 100.0 + error, &b) * 100.0 + error, &c) * M_PI / 648000.0 - 4.896618179206312657133e-15;
	x += (a + b / 60.0 + c / 3600.0) * M_PI / 180.0;
	return ((sgn == -1) ? -x : x);
}
- (IBAction)locate:(id)sender
// countryComboの値により，URLを変えた方が良い。maps.google.co.jpで外国を検索してもろくな地図が出ない。未完成。 //
{
	/* NSString *queryAddress = nil; */
	NSString *llStr = nil;
	// CFStringEncodings percentEncoding = kCFStringEncodingUTF8; は使えなくなった。 //

	if ((NO == [[[locateXForm cellAtIndex:0] stringValue] isEqualToString:@""]) && (NO == [[[locateYForm cellAtIndex:0] stringValue] isEqualToString:@""])) {
		double latDMS = [[locateXForm cellAtIndex:0] doubleValue];
		double lngDMS = [[locateYForm cellAtIndex:0] doubleValue];

		// queryAddress = [[NSString stringWithFormat:@"%f,%f(%@)",dtor(latDMS) * 180.0 * M_1_PI,dtor(lngDMS) * 180.0 * M_1_PI,[nameCombo objectValue]] stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet alphanumericCharacterSet]]; // stringByReplacingPercentEscapesUsingEncoding: との違いが判らない。 //
		llStr = [NSString stringWithFormat:@"&ll=%0.9f,%0.9f&t=h",dtor(latDMS) * 180.0 * M_1_PI,dtor(lngDMS) * 180.0 * M_1_PI];
	}
	/* else {
		queryAddress = [[NSString stringWithFormat:@"%@%@%@%@(%@)",[[prefectureForm cellAtIndex:0] stringValue], [[cityForm cellAtIndex:0] stringValue], [[townForm cellAtIndex:0] stringValue], [[addressForm cellAtIndex:0] stringValue],[nameCombo objectValue]] stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet alphanumericCharacterSet]]; // stringByReplacingPercentEscapesUsingEncoding: ではmapURLがnilになってしまう。 //
	} */

	NSString *queryAddress =[[NSString stringWithFormat:@"%@%@%@%@(%@)",[[prefectureForm cellAtIndex:0] stringValue], [[cityForm cellAtIndex:0] stringValue], [[townForm cellAtIndex:0] stringValue], [[addressForm cellAtIndex:0] stringValue],[nameCombo objectValue]] stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet alphanumericCharacterSet]]; // stringByReplacingPercentEscapesUsingEncoding: ではmapURLがnilになってしまう。 //
	if (nil != queryAddress) {
		/*  NSString *tempString = [NSString stringWithFormat:@"http://maps.google.co.jp/maps?q=%@&iwloc=A&hl=ja&f=l",queryAddress]; */
		NSMutableString *tempString = [NSMutableString stringWithFormat:@"http://maps.apple.com/maps?q=%@",(nil != queryAddress) ? queryAddress : @""];
		if (nil != llStr)
			[tempString appendString:llStr];
		NSURL *mapURL = [NSURL URLWithString:tempString];

        if ((nil != mapURL) && (NO == [[NSWorkspace sharedWorkspace] openURL:mapURL])) {
            NSAlert *alert = [[NSAlert alloc] init];
            // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
            [alert setMessageText:NSLocalizedString(@"Error!",@"")];
            [alert setInformativeText:tempString];
            [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
            [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
                // 何もやることがない //
            }];
            [alert release];
			/* NSBeginAlertSheet(NSLocalizedString(@"Error!",@""),nil,nil,nil,[self window],self,NULL,NULL,NULL,tempString,nil); */
        }
		else {
			if (nil == mapURL)
				NSLog(@"%@ mapURL is nil.",tempString);
		}
	}
	else
		NSLog(@"queeryAddress is nil.");
}

- (IBAction)saveTovCard:(id)sender
{
	NSSavePanel *sPanel = [NSSavePanel savePanel];

	[sPanel setDirectoryURL:nil];
	[sPanel setNameFieldStringValue:@""];
	[sPanel setCanSelectHiddenExtension:YES];
	[sPanel setAllowedFileTypes:[NSArray arrayWithObject:(NSString *)kUTTypeVCard]];
	[sPanel beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result)
	 {
		 if (NSModalResponseOK == result) {
			 AddressBookAdaptor *newInstance = [AddressBookAdaptor adaptor];
			 NSData *imageData = (nil != [pictureView image]) ? [[pictureView image] TIFFRepresentation] : nil;
			 NSData *vCardData = [newInstance vCardRepresentation:[self makeModel] image:imageData at:[indexCombo objectValue]];
			 
			 if (nil != vCardData)
				 [vCardData writeToURL:[sPanel URL] atomically:YES];
		 }
	 }
	 ];
}

- (void)saveToAddressBookFinal
{
	AddressBookAdaptor *newInstance = [AddressBookAdaptor adaptor];
	NSData *imageData = (nil != [pictureView image]) ? [[pictureView image] TIFFRepresentation] : nil;
	
    if (NO == [newInstance saveAperson:[self makeModel] image:imageData at:[indexCombo objectValue]]) {
        NSAlert *alert = [[NSAlert alloc] init];
        // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
        [alert setMessageText:NSLocalizedString(@"Error!",@"")];
        [alert setInformativeText:[indexCombo objectValue]];
        [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
        [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
            // 何もやることがない //
        }];
        [alert release];
		/* NSBeginAlertSheet(NSLocalizedString(@"Error!",@""),nil,nil,nil,[self window],self,NULL,NULL,NULL,[indexCombo objectValue],nil); */
    }
    else {
        NSAlert *alert = [[NSAlert alloc] init];
        // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
        [alert setMessageText:[indexCombo objectValue]];
        [alert setInformativeText:@"Saved"];
        [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
        [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
            // 何もやることがない //
        }];
        [alert release];
		/* NSBeginInformationalAlertSheet([indexCombo objectValue],nil,nil,nil,[self window],self,NULL,NULL,NULL,@"Saved.",nil); */
    }
}
- (IBAction)saveToAddressBook:(id)sender
{
	if ((nil != [indexCombo objectValue]) && (NO == [[indexCombo objectValue] isEqualToString:@""])) {
		AddressBookAdaptor *newInstance = [AddressBookAdaptor adaptor];

        if (YES == [newInstance isExistsAtAddressBook:[indexCombo objectValue]]) {
            NSAlert *alert = [[NSAlert alloc] init];
            [alert addButtonWithTitle:NSLocalizedString(@"Save",@"")];
            [alert addButtonWithTitle:NSLocalizedString(@"Don't Save",@"")];
            [alert setMessageText:NSLocalizedString(@"Allready exists at Address Book.",@"")];
            [alert setInformativeText:[indexCombo objectValue]];
            [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
            [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
                if (NSAlertFirstButtonReturn == result)
                    [self saveToAddressBookFinal];
            }];
            [alert release];
			/* NSBeginAlertSheet(NSLocalizedString(@"Allready exists at Address Book.",@""),NSLocalizedString(@"Save",@""),NSLocalizedString(@"Don't Save",@""),nil,[self window],self,NULL,@selector(saveToAddressBookDidEnd:returnCode:contextInfo:),NULL,[indexCombo objectValue],nil); */
        }
		else
			[self saveToAddressBookFinal];
	}
}

- (IBAction)saveToGroupInAddressBook:(id)sender
{
	if ((nil != [[self document] enrollDictionary]) && (0 < [[[self document] enrollDictionary] count])) {
		NSString *hint = (nil != [[self document] fileURL]) ? [[[[self document] fileURL] path] stringByDeletingPathExtension] : @"Group";

		[[groupnameForm cellAtIndex:0] setStringValue:hint];
		if (1 == [NSApp runModalForWindow:addressbookWindow]) {
			AddressBookAdaptor *newInstance = [AddressBookAdaptor adaptor];

			[addressbookProgress setIndeterminate:YES];
			[addressbookProgress setStyle:NSProgressIndicatorStyleBar];
			[addressbookProgress setUsesThreadedAnimation:YES];
			[addressbookProgress startAnimation:sender];
            if (NO == [newInstance saveAgroup:[[self document] fileURL] enroll:[[self document] enrollDictionary] group:[[groupnameForm cellAtIndex:0] stringValue] images:[[self document] valueForKey:EnrollImagesWrapperKey]]) {
                NSAlert *alert = [[NSAlert alloc] init];
                // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
                [alert setMessageText:NSLocalizedString(@"Error!",@"")];
                [alert setInformativeText:[[groupnameForm cellAtIndex:0] stringValue]];
                [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
                [alert beginSheetModalForWindow:[self window] completionHandler:^(NSInteger result) {
                    // 何もやることがない //
                }];
                [alert release];
				/* NSBeginAlertSheet(NSLocalizedString(@"Error!",@""),nil,nil,nil,[self window],self,NULL,NULL,NULL,[[groupnameForm cellAtIndex:0] stringValue],nil); */
            }
			[addressbookProgress stopAnimation:sender];
		}
		[addressbookWindow orderOut:sender];
	}
}

- (IBAction)groupOK:(id)sender {
	[NSApp stopModalWithCode:1];
}
- (IBAction)groupCancel:(id)sender {
	[NSApp stopModalWithCode:0];
}

@end
