/* PostalDictionaryServerProtocol.h created by 佐藤昭 on Mon 17-Aug-1998 */

/* NSConnection の ルートオブジェクトの名前は @"postalDictionaryServer" */
/* 通知
	NSConnectionDidDieNotification
	NSConnection がディアロケートされたか、その NSPort が無効化されたことが通知されたときにポストされます。この通知には、以下の内容がセットされます。
	通知オブジェクト	NSConnection オブジェクト
	このメッセージをポストした NSConnection オブジェクトは、以降の処理では使用できません。このため、すべてのレシーバは、NSConnection に関連するすべての通知に対して、自分自身の登録を抹消する必要があります。

	NSConnectionDidInitializeNotification
	initWithReceivePort:sendPort:（NSConnection の指定イニシャライザ）を使って NSConnection が初期化されたときにポストされます。この通知には、以下の内容がセットされます。
	通知オブジェクト	NSConnection オブジェクト
*/

/* PostalModelをクライアントから直接操作する必要がないので、PostalModelの説明は省略します。 */

@protocol PostalDictionaryServerProtocol

#define PostalDictionaryProtocolTypeKey @"Type"
#define PostalDictionaryProtocolFormKey @"Form"
#define PostalDictionaryProtocolPrefectureKey @"Prefecture"
#define PostalDictionaryProtocolPrefectureKanaKey @"PrefectureKana"
#define PostalDictionaryProtocolCityKey @"City"
#define PostalDictionaryProtocolCityKanaKey @"CityKana"
#define PostalDictionaryProtocolTownKey @"Town"
#define PostalDictionaryProtocolTownKanaKey @"TownKana"
#define PostalDictionaryProtocolAddressKey @"Address"
#define PostalDictionaryProtocolCompanyKey @"Company"
#define PostalDictionaryProtocolCompanyKanaKey @"CompanyKana"
#define PostalDictionaryProtocolPostOfficeKey @"PostOffice"
#define PostalDictionaryProtocolPostalNumberKey @"PostalNumber"
#define PostalDictionaryProtocolTownValue @"town"
#define PostalDictionaryProtocolCompanyValue @"company"

- (NSDictionary *)enablePrefectures;
// 利用可能な辞書を都道府県名で表現したNSDictionaryの配列を返します。キーは全国地方公共団体コードの上2桁のNSNumberです。値は都道府県名の文字列です。

- (NSDictionary *)workingPrefectures;
// 稼働中の郵便番号辞書を都道府県名で表現したNSDictionaryの配列を返します。キーは全国地方公共団体コードの上2桁のNSNumberです。値は都道府県名の文字列です。

- (NSUInteger)initWithPrefectures:(NSArray *)prefectures;
// prefecturesで指定された都道府県名の文字列の配列で特定される辞書を利用できる状態にします。郵便番号辞書の町域数を返します。辞書を利用できないときは、0を返します。

- (NSArray *)objectsForPostal:(NSUInteger)postalNumber;
// 7桁までの郵便番号postalNumberに該当する地域名・事業所名のNSDictionaryの配列を返します。postalNumberの数値は、例えば"012-0345"のときは"120345"というように指定してください。キー@"Type"の値が、@"town"のとき地域名が検索されたものとし、@"company"のとき事業所名が検索されたものとします。キー@"Form"の値は郵便番号の書式化された文字列,キー@"Prefecture"の値は都道府県名,キー@"City"の値は市区町村名,キー@"Town"の値は町域名です。地域名で検索されたとき、@"PrefectureKana"の値は都道府県名のカタカナ、@"CityKana"の値は市区町村名のカタカナ、@"TownKana"の値は町域名のカタカナです。事業所名で検索されたとき、@"Address"の値は番地等、@"Company"の値は事業所名、@"CompanyKana"の値は事業所名のカタカナ、@"PostOffice"の値は取扱郵便局名です。

- (NSArray *)postalForPrefecture:(NSString *)prefecture City:(NSString *)city Town:(NSString *)town;
// 都道府県名prefecture,市区町村名city,町域名townに該当するNSDictionaryの配列を返します。キー@"Type"の値は@"town"に限定される。キー@"PostalNumber"の値は郵便番号7桁の値をunsigned intとして扱ったNSNumberです。キー@"Form"の値は郵便番号の書式化された文字列,キー@"Prefecture"の値は都道府県名,キー@"City"の値は市区町村名,キー@"Town"の値は町域名です。@"PrefectureKana"の値は都道府県名のカタカナ、@"CityKana"の値は市区町村名のカタカナ、@"TownKana"の値は町域名のカタカナです。

- (NSArray *)postalForCompany:(NSString *)name;
// 大口事業所名nameに該当する郵便番号などのNSDictionaryの配列を返します。キー@"Type"の値は@"company"に限定される。キー@"PostalNumber"の値は郵便番号7桁の値をunsigned intとして扱ったNSNumberです。キー@"Form"の値は郵便番号の書式化された文字列,キー@"Prefecture"の値は都道府県名,キー@"City"の値は市区町村名,キー@"Town"の値は町域名です。@"Address"の値は番地等、@"Company"の値は事業所名、@"CompanyKana"の値は事業所名のカタカナ、@"PostOffice"の値は取扱郵便局名です。

- (NSString *)dictionaryDate;
// 稼働中の郵便番号辞書の更新日付を返します。

- (NSString *)dictionaryName;
// 稼働中の郵便番号辞書を都道府県名で表現した文字列を返します。末尾にスペース文字が1つ附加されることに注意してください。

- (NSUInteger)towns;
// 稼働中の郵便番号辞書の町域数を返します。

- (NSString *)postalExtension;
// 郵便番号辞書のアーカイブファイルの拡張子を返します。

- (BOOL)writeToFile:(NSString *)fullpath atomically:(BOOL)flag;
// 稼働中の郵便番号辞書を書式化した文字列をfullpathで指定されたパス位置にあるファイルに書き込みます。正常に書き込みを行うことができたときにはYESを返し、書き込みに失敗したときはNOを返します。

- (NSString *)description;
// 稼働中の郵便番号辞書を書式化した文字列を返します。

- (NSArray *)allPostals;
// 稼働中の郵便番号辞書の郵便番号の配列を返します。配列の各要素は、郵便番号7桁の値をunsigned intとして扱ったNSNumberです。

- (NSDictionary *)postalDictionary;
// 稼働中の郵便番号辞書を返します。(郵便番号7桁+同一番号識別用の2桁) = 9桁のNSNumberをキーとして、値はPostalModelです。

- (NSArray *)allModels;
// 稼働中の郵便番号辞書の値PostalModelの配列を返します。

- (NSEnumerator *)postalEnumerator;
// 稼働中の郵便番号辞書のキーのNSEnumeratorを返します。キーは(郵便番号7桁+同一番号識別用の2桁) = 9桁のNSNumberです。

- (NSEnumerator *)modelsEnumerator;
// 稼働中の郵便番号辞書の値PostalModelのNSEnumeratorを返します。

- (NSString *)separator;
// 郵便番号書式中の区切り文字

- (NSString *)zipFormat;
// 郵便番号書式

- (NSString *)otherTown;
// "以下に掲載がない場合"を返す。

@end
