/* PostalFind.m created by me on Wed 30-Jun-1999 */

#import "PostalFind.h"
#ifdef __APPLE__
#define SERVER_PORT 15550
#endif
#define SERVER_NAME @"com.SatoAkira.postalDictionaryServer"

@implementation PostalFind

+ (PostalFind *)postalFind {
	PostalFind *newInstance = [[self alloc] init];

	return [newInstance autorelease];
}

- (instancetype)init {
	self = [super init];
	theProxy = nil;
	delegate = nil;
	registeredName = [[NSString alloc] initWithString:SERVER_NAME];
	separator = nil;
	otherTown = nil;
	return self;
}

- (void)dealloc {
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];

	[nc removeObserver:self];
	[theProxy release]; // はしない方が良い //
	[registeredName release];
	[separator release];
	[otherTown release];
	[super dealloc];
}

- (BOOL)postalSet {
	NSConnection *theConnection;
#ifdef __APPLE__
	NSString *hostName = nil; // OPENSTEPのような"*"は使えない。NSSocketPortが使われたサーバーに対してはhostNameを指定することができるが、serverにDidDieNotificationを送れない。nilでもNSSocketPortが使われたサーバーに接続できるので、serverにDidDieNotificationを送るにはhostNameをnillにしなければならない。NSSocketPortが使われたサーバーに接続しているときにはクライアント側にDidDieNotificationが通知されない。クライアント側でその必要があるときにはサーバーはMachPortを使わなければならない。MachPortの場合にはhostNameはnilでなければならない。 //
	NSPort *sendPort = [[NSMachBootstrapServer sharedInstance] portForName:registeredName host:hostName];

	if (nil == sendPort) {
        // This will succeed (if host exists), even when there is no server
        // on the other end, since the connect() is done lazily (arguably wrong),
        // when first message is sent.
		sendPort = [[NSSocketPort alloc] initRemoteWithTCPPort:SERVER_PORT host:hostName];
		if (nil == sendPort) {
			NSLog(@"client: could not look up server\n");
			theProxy = nil;
		}
		else {
NS_DURING
			theConnection = [[NSConnection alloc] initWithReceivePort:(NSPort *)[[sendPort class] port] sendPort:sendPort];
			theProxy = [[theConnection rootProxy] retain];
NS_HANDLER
			theProxy = nil;
NS_ENDHANDLER
		}
	}
	else {
		if (nil != (theConnection = [NSConnection connectionWithReceivePort:nil sendPort:sendPort]))
			theProxy = [[theConnection rootProxy] retain];  // theProxyはNSDistantObject クラス //
		else
			theProxy = nil;
	}
#else
	NSString *hostName = @"*";

	if (nil != (theConnection = [NSConnection connectionWithRegisteredName:registeredName host:hostName]))
		theProxy = [[theConnection rootProxy] retain];
	else
		theProxy = nil;
#endif
	if (nil != theProxy) {
		NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];

		[theProxy setProtocolForProxy:@protocol(PostalDictionaryServerProtocol)];
		[nc addObserver:self selector:@selector(connectionDidDie:) name:NSConnectionDidDieNotification object:theConnection];
		[nc addObserver:self selector:@selector(connectionDidInitialize:) name:NSConnectionDidInitializeNotification object:theConnection];
		[separator release];
		separator = [[theProxy separator] copy];
		[otherTown release];
		otherTown = [[theProxy otherTown] copy];
		return YES;
	}
	else
		return NO;
}

- (void)connectionDidDie:(NSNotification *)notification {
	// theConnectionは、自動的に無効化されるが、nilにはならない。 //
	// 以後は[theProxy connectionForProxy]も無効化されるので、これで調べる方法が良い。 //

    NSAlert *alert = [[NSAlert alloc] init];
    // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
    [alert setMessageText:NSStringFromClass([self class])];
    [alert setInformativeText:[notification name]];
    [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
    [alert runModal];
    [alert release];
	if ((nil != delegate) && (YES == [delegate respondsToSelector:@selector(postalDidDie:)]))
		[delegate postalDidDie:notification];
	// これ以後はdelegateによって自分自身を解放されてしまうときもあるから何かしてはいけない。 //
}

- (void)connectionDidInitialize:(NSNotification *)notification {
	// userInfoは無い //
    NSAlert *alert = [[NSAlert alloc] init];
    // [alert addButtonWithTitle:@"OK"]; // // 無くても良い //
    [alert setMessageText:NSStringFromClass([self class])];
    [alert setInformativeText:[notification name]];
    [alert setAlertStyle:NSAlertStyleWarning]; // NSWarningAlertStyle,NSInformationalAlertStyle,NSCriticalAlertStyle //
    [alert runModal];
    [alert release];
	if ((nil != delegate) && (YES == [delegate respondsToSelector:@selector(postalDidInitialize:)]))
		[delegate postalDidInitialize:self];
}

- (void)setDelegate:(id)anObject {
	delegate = [anObject retain];
}

- (NSArray *)findWithZip:(NSString *)code {
	if (nil != theProxy) {
		NSInteger zip1,zip2;
		NSScanner *aScanner = [NSScanner scannerWithString:code];

		if ((YES == [aScanner scanInteger:&zip1]) && (YES == [aScanner scanString:separator intoString:NULL]) && (YES == [aScanner scanInteger:&zip2]))
			return [theProxy objectsForPostal:(NSUInteger)(zip1 * 10000 + zip2)];
		else
			return [NSArray array];
	}
	else
		return [NSArray array];
}

- (NSArray *)findWithPrefecture:(NSString *)prefecture City:(NSString *)city Town:(NSString *)town {
	if (nil != theProxy)
		return [theProxy postalForPrefecture:prefecture City:city Town:town];
	else
		return [NSArray array];
}

- (NSArray *)findWithCompany:(NSString *)name {
	if (nil != theProxy)
		return [theProxy postalForCompany:name];
	else
		return [NSArray array];
}

- (NSString *)registeredName; {
	return registeredName;
}
- (NSString *)separator {
	return separator;
}
- (NSString *)otherTown {
	return otherTown;
}
- (id)postalProxy {
	return theProxy;
}
- (id)delegate {
	return delegate;
}
- (NSString *)dictionaryName {
	return (nil != theProxy) ? [theProxy dictionaryName] : [NSString string];
}
- (NSString *)dictionaryDate {
	return (nil != theProxy) ? [theProxy dictionaryDate] : [NSString string];
}
- (NSDictionary *)workingPrefectures {
	return (nil != theProxy) ? [theProxy workingPrefectures] : [NSDictionary dictionary];
}
- (NSDictionary *)enablePrefectures {
	return (nil != theProxy) ? [theProxy enablePrefectures] : [NSDictionary dictionary];
}
- (NSUInteger)setPrefectures:(NSArray *)prefectures {
	return (nil != theProxy) ? [theProxy initWithPrefectures:prefectures] : 0;
}

@end
