//
//  Report.m
//  Enroll
//
//  Created by 佐藤 昭 on Wed Jul 17 2002.
//  Copyright (c) 2002 SatoAkira. All rights reserved.
//

#import "Report.h"
#import "MainDocument.h"

@interface Report ()
@property IBOutlet NSBox *sortBox;
@property IBOutlet NSMatrix *sortMatrix;
@property BOOL releasedWhenClosed;
@end

@implementation Report

+ (Report *)report {
	Report *newInstance = [[self alloc] init];
	newInstance.releasedWhenClosed = YES; // ファクトリーメソッドから作る場合にはYESにする。 //
	return newInstance;
}

- (instancetype)init {
	if (nil != (self = [super initWithWindowNibName:NSStringFromClass([self class])])) {
		self.releasedWhenClosed = NO;
	}
	return self;
}
- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[self window] setDelegate:nil];
	[[NSNotificationCenter defaultCenter] postNotificationName:NSWindowWillCloseNotification object:self];
	[super dealloc];
}
- (void)windowDidLoad {
	[searchBox setTitle:NSLocalizedString(@"SearchItemAndString",@"検索項目・文字列")];
	[searchPopUp removeAllItems];
	[searchPopUp addItemWithTitle:NSLocalizedString(@"AbstractAll",@"すべて")];
	[searchPopUp addItemWithTitle:NSLocalizedString(IndexKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(RootIndexKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(NameKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(CountryKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(ZipKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(PrefectureKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(CityKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(TownKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(AddressKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(TelNumberKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(FaxNumberKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(PHSNumberKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(EmailKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(URLNameKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(TypeKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(BirthDayKey,@"")];
	[searchPopUp addItemWithTitle:NSLocalizedString(OthersKey,@"")];
	[[searchField cell] setSendsActionOnEndEditing:YES];
	[searchField setStringValue:@""];
	[self.sortBox setTitle:NSLocalizedString(@"SortItem",@"ソート項目")];
	[[self.sortMatrix cellAtRow:0 column:0] setTitle:NSLocalizedString(@"Index",@"")];
	[[self.sortMatrix cellAtRow:0 column:1] setTitle:NSLocalizedString(@"Name",@"")];
	[[self.sortMatrix cellAtRow:0 column:2] setTitle:NSLocalizedString(@"Abode",@"住所")];
	[[self.sortMatrix cellAtRow:0 column:3] setTitle:NSLocalizedString(@"Type",@"")];
	[[self.sortMatrix cellAtRow:0 column:4] setTitle:NSLocalizedString(@"BirthDay",@"")];
	[[self.sortMatrix cellAtRow:0 column:5] setTitle:NSLocalizedString(@"NameKana",@"ふりがな")];
	[underlineSwitch setTitle:NSLocalizedString(@"Underline",@"アンダーライン")];
	[reportTextView setUsesFindPanel:YES];
	[[self window] setDelegate:self];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:[[self document] windowController]];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(contextDidChange:) name:ContextDidChangeNotification object:[self document]];
}
- (void)windowWillClose:(NSNotification *)aNotification {
	[[self window] orderOut:aNotification];
	if (YES == self.releasedWhenClosed)
		[self release];
}

- (NSAttributedString *)reportString:(NSString *)search item:(NSString *)itemString sort:(NSString *)sortString underline:(NSInteger)ulState {
	NSDictionary *eDic = [[self document] enrollDictionary];

	if (nil != eDic) {
		NSRange range;
		NSMutableString *tempStr;
		NSAttributedString *tempAttrStr;
		NSDictionary *aModel;
		id searchObj;
		NSArray *arrayValue;
		NSFont *titleFont = [NSFont fontWithName:NSLocalizedString(@"FixedMinBoldFont",@"HiraMinPro-W6") size:13.0];
		NSFont *font = [NSFont fontWithName:NSLocalizedString(@"FixedMinFont",@"") size:9.0];
		NSString *empty = @"";
		NSMutableAttributedString *rStr = [[NSMutableAttributedString alloc] initWithString:empty attributes:nil];
		NSMutableDictionary *sAttr = [NSMutableDictionary dictionaryWithObjectsAndKeys:titleFont, NSFontAttributeName, [NSColor textColor], NSForegroundColorAttributeName, nil];
		NSString *itemKey = (YES == [itemString isEqualToString:NSLocalizedString(@"AbstractAll",@"")]) ? nil : [[self document] reverseLocalizedString:itemString];
		BOOL itemIsIndex = [itemString isEqualToString:NSLocalizedString(IndexKey,@"")];
		NSString *sortItem = [[self document] reverseLocalizedString:sortString];
		NSUInteger sortKind = 0;
		NSMutableDictionary *sDic = [NSMutableDictionary dictionary];

		[rStr beginEditing];
		tempAttrStr = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\t%@ %@\n",(nil != [[self document] fileURL]) ? [[[self document] fileURL] path] : NSLocalizedString(@"Untitled",@""),NSLocalizedString(@"List",@"リスト")] attributes:sAttr];
		[rStr appendAttributedString:tempAttrStr];
		[tempAttrStr release];
		[sAttr setObject:font forKey:NSFontAttributeName];
		NSDateFormatter *dateTimeFormatter = [[NSDateFormatter alloc] init];
		[dateTimeFormatter setFormatterBehavior:NSDateFormatterBehavior10_4];
		[dateTimeFormatter setCalendar:[NSCalendar autoupdatingCurrentCalendar]];
		[dateTimeFormatter setDateStyle:NSDateFormatterFullStyle];
		[dateTimeFormatter setTimeStyle:NSDateFormatterFullStyle];
		tempAttrStr = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\t\t%@\n",[dateTimeFormatter stringFromDate:[NSDate date]]] attributes:sAttr];
		[rStr appendAttributedString:tempAttrStr];
		[dateTimeFormatter release];
		[tempAttrStr release];
		if (nil != itemKey) {
			tempAttrStr = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@:%@\t%@:%@\n",NSLocalizedString(@"SearchItem",@"検索項目"),itemString,NSLocalizedString(@"FindString",@"検索文字列"),search]  attributes:sAttr];
			[rStr appendAttributedString:tempAttrStr];
			[tempAttrStr release];
		}

		[sAttr setObject:[NSNumber numberWithInt:(NSControlStateValueOn == ulState) ? 1 : 0] forKey:NSUnderlineStyleAttributeName];

		if (NO == [sortItem isEqualToString:IndexKey]) {
			if (YES == [sortString isEqualToString:NSLocalizedString(@"Abode",@"住所")])
				sortKind = 2;
			else {
				if (YES == [sortItem isEqualToString:BirthDayKey])
					sortKind = 4;
				else {
					if (YES == [sortItem isEqualToString:TypeKey])
						sortKind = 3;
					else {
						if (YES == [sortItem isEqualToString:NameKanaKey])
							sortKind = 5;
						else {
							if (YES == [sortItem isEqualToString:NameKey])
								sortKind = 1;
							else
								sortKind = 0;
						}
					}
				}
			}
			// NSLog(@"sortItem=%@ kind=%lu",sortItem,sortKind); //
		}
		else
			sortKind = 0;
        NSUInteger j;
        for (NSString *key in [eDic keyEnumerator]) {
			aModel = [eDic objectForKey:key];
			if (nil != itemKey) { // 検索 //
				if (YES == itemIsIndex)
					range = [key rangeOfString:search];
				else {
					if (nil != (searchObj = [aModel objectForKey:itemKey])) {
						if (YES == [searchObj isKindOfClass:[NSArray class]]) {
							for (range = NSMakeRange(0,0), j = 0; j < [searchObj count]; j++) {
								range = [[searchObj objectAtIndex:j] rangeOfString:search];
								if (0 < range.length)
									break;
							} 
						}
						else
							range = [[aModel objectForKey:itemKey] rangeOfString:search];
					}
					else
						range = NSMakeRange(0,0);
				}
			}
			if ((nil == itemKey) || (0 < range.length)) {
				UniChar nonCharUniChar[] = {(UniChar)0xfffd};
				NSString *invalidStr = [NSString stringWithCharacters:nonCharUniChar length:1];
				switch (sortKind) { // ソートの準備 //
				case 0: // コード //
					[sDic setObject:key forKey:key];
					break;
					case 1: // 氏名 //
						if (nil != [aModel objectForKey:sortItem])
							[sDic setObject:[aModel objectForKey:sortItem] forKey:key];
						else
							[sDic setObject:invalidStr forKey:key];
						break;
				case 2: // 住所 //
					[sDic setObject:[NSString stringWithFormat:@"%@%@%@%@",(nil != [aModel objectForKey:PrefectureKey]) ? [aModel objectForKey:PrefectureKey] : empty,(nil != [aModel objectForKey:CityKey]) ? [aModel objectForKey:CityKey] : empty,(nil != [aModel objectForKey:TownKey]) ? [aModel objectForKey:TownKey] : empty,(nil != [aModel objectForKey:AddressKey]) ? [aModel objectForKey:AddressKey] : empty] forKey:key];
					break;
					case 3: // 分類 //
						if (nil != [aModel objectForKey:sortItem])
							[sDic setObject:[aModel objectForKey:sortItem] forKey:key];
						else
							[sDic setObject:invalidStr forKey:key];
						break;
					case 4: // 誕生日 //
                        if ((nil != [aModel objectForKey:BirthDayKey]) && (NO == [[aModel objectForKey:BirthDayKey] isEqualToString:empty])) {
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_10
							NSCalendar *calendar = [[NSCalendar alloc] initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
                            NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
                            [formatter setFormatterBehavior:NSDateFormatterBehavior10_4];
							[formatter setCalendar:calendar];
							[formatter setDateFormat:@"yyyy-MM-dd"];
							NSDate *date = [formatter dateFromString:[aModel objectForKey:BirthDayKey]];
							[sDic setObject:(nil != date) ? date : [NSDate distantFuture] forKey:key];
							[calendar release];
							[formatter release];
#else
                            [sDic setObject:[NSDate dateWithNaturalLanguageString:[aModel objectForKey:BirthDayKey]] forKey:key];
#endif
                        }
                        else
                            [sDic setObject:[NSDate distantFuture] forKey:key];
                        break;
				case 5: // ふりがな //
						if (nil != [aModel objectForKey:sortItem])
							[sDic setObject:[aModel objectForKey:sortItem] forKey:key];
						else
							[sDic setObject:invalidStr forKey:key];
					break;
				default:
					if (nil != [aModel objectForKey:sortItem])
						[sDic setObject:[aModel objectForKey:sortItem] forKey:key];
					else
						[sDic setObject:invalidStr forKey:key];
					break;
				}
			}
		}
		if (0 < [sDic count]) {
			NSArray *sortedKeys = [sDic keysSortedByValueUsingSelector:@selector(compare:)]; // ソート //

            for (NSString *key in sortedKeys) { // 表示 //
				aModel = [eDic objectForKey:key];
				tempStr = [NSMutableString string];
				[tempStr appendString:key];
				[tempStr appendFormat:@" %@",(nil != [aModel objectForKey:NameKey]) ? [aModel objectForKey:NameKey] : empty];
				[tempStr appendFormat:@" %@",(nil != [aModel objectForKey:ZipKey]) ? [aModel objectForKey:ZipKey] : empty];
				[tempStr appendFormat:@" %@",(nil != [aModel objectForKey:PrefectureKey]) ? [aModel objectForKey:PrefectureKey] : empty];
				[tempStr appendString:(nil != [aModel objectForKey:CityKey]) ? [aModel objectForKey:CityKey] : empty];
				[tempStr appendString:(nil != [aModel objectForKey:TownKey]) ? [aModel objectForKey:TownKey] : empty];
				[tempStr appendString:(nil != [aModel objectForKey:AddressKey]) ? [aModel objectForKey:AddressKey] : empty];
				[tempStr appendFormat:@" %@",(nil != [aModel objectForKey:TelNumberKey]) ? [aModel objectForKey:TelNumberKey] : empty];
				[tempStr appendFormat:@" %@",(nil != [aModel objectForKey:FaxNumberKey]) ? [aModel objectForKey:FaxNumberKey] : empty];
				[tempStr appendFormat:@" %@",(nil != [aModel objectForKey:PHSNumberKey]) ? [aModel objectForKey:PHSNumberKey] : empty];
				if ((nil != (arrayValue = [aModel objectForKey:EmailKey])) && (0 < [arrayValue count])) {
                    for (NSString *aStr in arrayValue)
                        [tempStr appendFormat:@" %@",aStr];
				}
				if ((nil != (arrayValue = [aModel objectForKey:URLNameKey])) && (0 < [arrayValue count])) {
                    for (NSString *aStr in arrayValue)
                        [tempStr appendFormat:@" %@",aStr];
				}
				[tempStr appendFormat:@" %@",(nil != [aModel objectForKey:TypeKey]) ? [aModel objectForKey:TypeKey] : empty];
				[tempStr appendFormat:@" %@",(nil != [aModel objectForKey:BirthDayKey]) ? [aModel objectForKey:BirthDayKey] : empty];
				[tempStr appendFormat:@" %@\n",(nil != [aModel objectForKey:OthersKey]) ? [aModel objectForKey:OthersKey] : empty];
				tempAttrStr = [[NSAttributedString alloc] initWithString:tempStr attributes:sAttr];
				[rStr appendAttributedString:tempAttrStr];
				[tempAttrStr release];
			}
		}
		[rStr endEditing];
		return [rStr autorelease];
	}
	else
		return nil;
}

- (IBAction)showReport:(id)sender {
	NSString *sStr = [searchField stringValue];
	NSAttributedString *attrStr = [self reportString:sStr item:[searchPopUp titleOfSelectedItem] sort:[[self.sortMatrix selectedCell] title] underline:[underlineSwitch state]];

	if (nil != attrStr) {
				/* NSRect windowRect = [[self window] frame];
				float minWidth = [[self window] minSize].width;
				float maxWidth = windowRect.size.width - [reportTextView bounds].size.width + [attrStr size].width + 2.0 * [[reportTextView textContainer] lineFragmentPadding];
				float newWidth = (maxWidth > minWidth) ? maxWidth : minWidth;

				windowRect.size.width = newWidth;
				[[self window] setFrame:windowRect display:YES]; */
		[[reportTextView textStorage] setAttributedString:attrStr];
	}
	else
		[reportTextView setString:@""];
}

- (IBAction)printDocument:(id)sender {
	if (0 < reportTextView.textStorage.length) {
		NSTextView *renderingTextView = [[NSTextView alloc] initWithFrame:NSMakeRect(0.0,0.0,NSWidth(reportTextView.bounds),NSHeight(reportTextView.bounds))];
		renderingTextView.appearance = [NSAppearance  appearanceNamed:NSAppearanceNameAqua]; // デフォルトでAquaになっているのでやらなくても良い。 //
		[renderingTextView.textStorage setAttributedString:reportTextView.textStorage];
		NSPrintOperation *op = [NSPrintOperation printOperationWithView:renderingTextView printInfo:[self.document printInfo]];
		NSPrintPanel *printPanel = [op printPanel]; // [printPanel options]のデフォルト値は131075 //
		[printPanel setOptions:[printPanel options] | NSPrintPanelShowsPaperSize | NSPrintPanelShowsOrientation | NSPrintPanelShowsScaling]; // NSPrintPanelShowsPageRange:デフォルト NSPrintPanelShowsPrintSelection:すべて,開始の下へ選択部分を加えるが何かが足りないようでうまくいかない。 NSPrintPanelShowsPageSetupAccessory:PageLayoutを真ん中のポップアップに加える。 //
		[op runOperationModalForWindow:self.window delegate:self didRunSelector:NULL contextInfo:NULL]; // didRunSelectorでやることはprintInfoを保存するくらいのことしか無いので何もしないことにする。 //
		[renderingTextView release];
	}
}

- (void)setSearchItem:(id)sender {
	[self showReport:searchField];
}
- (void)setSortItem:(id)sender {
	[self showReport:searchField];
}
- (void)contextDidChange:(NSNotification *)notification {
	[self showReport:searchField];
}

@end
